from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import dataflow_radix2

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  reg [32-1:0] din0re;
  reg [32-1:0] din0im;
  reg [32-1:0] din1re;
  reg [32-1:0] din1im;
  reg [32-1:0] cnstre;
  reg [32-1:0] cnstim;
  wire [32-1:0] dout1re;
  wire [32-1:0] dout1im;
  wire [32-1:0] dout0re;
  wire [32-1:0] dout0im;

  radix2
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .din0re(din0re),
    .din0im(din0im),
    .din1re(din1re),
    .din1im(din1im),
    .cnstre(cnstre),
    .cnstim(cnstim),
    .dout1re(dout1re),
    .dout1im(dout1im),
    .dout0re(dout0re),
    .dout0im(dout0im)
  );

  reg reset_done;

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    reset_done = 0;
    din0re = 2;
    din0im = 0;
    din1re = 1;
    din1im = 0;
    cnstre = 0;
    cnstim = 1;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    reset_done = 1;
    @(posedge CLK);
    #1;
    #10000;
    $finish;
  end

  reg [32-1:0] send_fsm;
  localparam send_fsm_init = 0;
  localparam send_fsm_1 = 1;
  localparam send_fsm_2 = 2;
  localparam send_fsm_3 = 3;
  localparam send_fsm_4 = 4;
  localparam send_fsm_5 = 5;
  localparam send_fsm_6 = 6;
  localparam send_fsm_7 = 7;
  localparam send_fsm_8 = 8;
  localparam send_fsm_9 = 9;
  localparam send_fsm_10 = 10;
  localparam send_fsm_11 = 11;

  always @(posedge CLK) begin
    if(RST) begin
      send_fsm <= send_fsm_init;
    end else begin
      case(send_fsm)
        send_fsm_init: begin
          if(reset_done) begin
            send_fsm <= send_fsm_1;
          end 
        end
        send_fsm_1: begin
          send_fsm <= send_fsm_2;
        end
        send_fsm_2: begin
          send_fsm <= send_fsm_3;
        end
        send_fsm_3: begin
          send_fsm <= send_fsm_4;
        end
        send_fsm_4: begin
          send_fsm <= send_fsm_5;
        end
        send_fsm_5: begin
          send_fsm <= send_fsm_6;
        end
        send_fsm_6: begin
          send_fsm <= send_fsm_7;
        end
        send_fsm_7: begin
          send_fsm <= send_fsm_8;
        end
        send_fsm_8: begin
          send_fsm <= send_fsm_9;
        end
        send_fsm_9: begin
          send_fsm <= send_fsm_10;
        end
        send_fsm_10: begin
          send_fsm <= send_fsm_11;
        end
        send_fsm_11: begin
          $finish;
        end
      endcase
    end
  end


endmodule



module radix2
(
  input CLK,
  input RST,
  input [32-1:0] din0re,
  input [32-1:0] din0im,
  input [32-1:0] din1re,
  input [32-1:0] din1im,
  input [32-1:0] cnstre,
  input [32-1:0] cnstim,
  output [32-1:0] dout1re,
  output [32-1:0] dout1im,
  output [32-1:0] dout0re,
  output [32-1:0] dout0im
);

  reg [32-1:0] _tmp_data_0;
  reg _tmp_valid_0;
  wire _tmp_ready_0;
  assign _tmp_ready_0 = (_tmp_ready_10 || !_tmp_valid_10) && _tmp_valid_0;
  reg [32-1:0] _tmp_data_1;
  reg _tmp_valid_1;
  wire _tmp_ready_1;
  assign _tmp_ready_1 = (_tmp_ready_11 || !_tmp_valid_11) && _tmp_valid_1;
  reg [32-1:0] _tmp_data_2;
  reg _tmp_valid_2;
  wire _tmp_ready_2;
  assign _tmp_ready_2 = (_tmp_ready_6 || !_tmp_valid_6) && (_tmp_valid_2 && _tmp_valid_4) && ((_tmp_ready_8 || !_tmp_valid_8) && (_tmp_valid_2 && _tmp_valid_5));
  reg [32-1:0] _tmp_data_3;
  reg _tmp_valid_3;
  wire _tmp_ready_3;
  assign _tmp_ready_3 = (_tmp_ready_7 || !_tmp_valid_7) && (_tmp_valid_3 && _tmp_valid_5) && ((_tmp_ready_9 || !_tmp_valid_9) && (_tmp_valid_3 && _tmp_valid_4));
  reg [32-1:0] _tmp_data_4;
  reg _tmp_valid_4;
  wire _tmp_ready_4;
  assign _tmp_ready_4 = (_tmp_ready_6 || !_tmp_valid_6) && (_tmp_valid_2 && _tmp_valid_4) && ((_tmp_ready_9 || !_tmp_valid_9) && (_tmp_valid_3 && _tmp_valid_4));
  reg [32-1:0] _tmp_data_5;
  reg _tmp_valid_5;
  wire _tmp_ready_5;
  assign _tmp_ready_5 = (_tmp_ready_7 || !_tmp_valid_7) && (_tmp_valid_3 && _tmp_valid_5) && ((_tmp_ready_8 || !_tmp_valid_8) && (_tmp_valid_2 && _tmp_valid_5));
  wire [32-1:0] _tmp_data_6;
  wire _tmp_valid_6;
  wire _tmp_ready_6;
  wire [64-1:0] _tmp_odata_6;
  reg [64-1:0] _tmp_data_reg_6;
  assign _tmp_data_6 = _tmp_data_reg_6;
  wire _tmp_ovalid_6;
  reg _tmp_valid_reg_6;
  assign _tmp_valid_6 = _tmp_valid_reg_6;
  wire _tmp_enable_6;
  wire _tmp_update_6;
  assign _tmp_enable_6 = (_tmp_ready_6 || !_tmp_valid_6) && (_tmp_ready_2 && _tmp_ready_4) && (_tmp_valid_2 && _tmp_valid_4);
  assign _tmp_update_6 = _tmp_ready_6 || !_tmp_valid_6;

  multiplier_0
  mul6
  (
    .CLK(CLK),
    .RST(RST),
    .update(_tmp_update_6),
    .enable(_tmp_enable_6),
    .valid(_tmp_ovalid_6),
    .a(_tmp_data_2),
    .b(_tmp_data_4),
    .c(_tmp_odata_6)
  );

  assign _tmp_ready_6 = (_tmp_ready_24 || !_tmp_valid_24) && (_tmp_valid_6 && _tmp_valid_7);
  wire [32-1:0] _tmp_data_7;
  wire _tmp_valid_7;
  wire _tmp_ready_7;
  wire [64-1:0] _tmp_odata_7;
  reg [64-1:0] _tmp_data_reg_7;
  assign _tmp_data_7 = _tmp_data_reg_7;
  wire _tmp_ovalid_7;
  reg _tmp_valid_reg_7;
  assign _tmp_valid_7 = _tmp_valid_reg_7;
  wire _tmp_enable_7;
  wire _tmp_update_7;
  assign _tmp_enable_7 = (_tmp_ready_7 || !_tmp_valid_7) && (_tmp_ready_3 && _tmp_ready_5) && (_tmp_valid_3 && _tmp_valid_5);
  assign _tmp_update_7 = _tmp_ready_7 || !_tmp_valid_7;

  multiplier_1
  mul7
  (
    .CLK(CLK),
    .RST(RST),
    .update(_tmp_update_7),
    .enable(_tmp_enable_7),
    .valid(_tmp_ovalid_7),
    .a(_tmp_data_3),
    .b(_tmp_data_5),
    .c(_tmp_odata_7)
  );

  assign _tmp_ready_7 = (_tmp_ready_24 || !_tmp_valid_24) && (_tmp_valid_6 && _tmp_valid_7);
  wire [32-1:0] _tmp_data_8;
  wire _tmp_valid_8;
  wire _tmp_ready_8;
  wire [64-1:0] _tmp_odata_8;
  reg [64-1:0] _tmp_data_reg_8;
  assign _tmp_data_8 = _tmp_data_reg_8;
  wire _tmp_ovalid_8;
  reg _tmp_valid_reg_8;
  assign _tmp_valid_8 = _tmp_valid_reg_8;
  wire _tmp_enable_8;
  wire _tmp_update_8;
  assign _tmp_enable_8 = (_tmp_ready_8 || !_tmp_valid_8) && (_tmp_ready_2 && _tmp_ready_5) && (_tmp_valid_2 && _tmp_valid_5);
  assign _tmp_update_8 = _tmp_ready_8 || !_tmp_valid_8;

  multiplier_2
  mul8
  (
    .CLK(CLK),
    .RST(RST),
    .update(_tmp_update_8),
    .enable(_tmp_enable_8),
    .valid(_tmp_ovalid_8),
    .a(_tmp_data_2),
    .b(_tmp_data_5),
    .c(_tmp_odata_8)
  );

  assign _tmp_ready_8 = (_tmp_ready_25 || !_tmp_valid_25) && (_tmp_valid_8 && _tmp_valid_9);
  wire [32-1:0] _tmp_data_9;
  wire _tmp_valid_9;
  wire _tmp_ready_9;
  wire [64-1:0] _tmp_odata_9;
  reg [64-1:0] _tmp_data_reg_9;
  assign _tmp_data_9 = _tmp_data_reg_9;
  wire _tmp_ovalid_9;
  reg _tmp_valid_reg_9;
  assign _tmp_valid_9 = _tmp_valid_reg_9;
  wire _tmp_enable_9;
  wire _tmp_update_9;
  assign _tmp_enable_9 = (_tmp_ready_9 || !_tmp_valid_9) && (_tmp_ready_3 && _tmp_ready_4) && (_tmp_valid_3 && _tmp_valid_4);
  assign _tmp_update_9 = _tmp_ready_9 || !_tmp_valid_9;

  multiplier_3
  mul9
  (
    .CLK(CLK),
    .RST(RST),
    .update(_tmp_update_9),
    .enable(_tmp_enable_9),
    .valid(_tmp_ovalid_9),
    .a(_tmp_data_3),
    .b(_tmp_data_4),
    .c(_tmp_odata_9)
  );

  assign _tmp_ready_9 = (_tmp_ready_25 || !_tmp_valid_25) && (_tmp_valid_8 && _tmp_valid_9);
  reg [32-1:0] _tmp_data_10;
  reg _tmp_valid_10;
  wire _tmp_ready_10;
  assign _tmp_ready_10 = (_tmp_ready_12 || !_tmp_valid_12) && _tmp_valid_10;
  reg [32-1:0] _tmp_data_11;
  reg _tmp_valid_11;
  wire _tmp_ready_11;
  assign _tmp_ready_11 = (_tmp_ready_13 || !_tmp_valid_13) && _tmp_valid_11;
  reg [32-1:0] _tmp_data_12;
  reg _tmp_valid_12;
  wire _tmp_ready_12;
  assign _tmp_ready_12 = (_tmp_ready_14 || !_tmp_valid_14) && _tmp_valid_12;
  reg [32-1:0] _tmp_data_13;
  reg _tmp_valid_13;
  wire _tmp_ready_13;
  assign _tmp_ready_13 = (_tmp_ready_15 || !_tmp_valid_15) && _tmp_valid_13;
  reg [32-1:0] _tmp_data_14;
  reg _tmp_valid_14;
  wire _tmp_ready_14;
  assign _tmp_ready_14 = (_tmp_ready_16 || !_tmp_valid_16) && _tmp_valid_14;
  reg [32-1:0] _tmp_data_15;
  reg _tmp_valid_15;
  wire _tmp_ready_15;
  assign _tmp_ready_15 = (_tmp_ready_17 || !_tmp_valid_17) && _tmp_valid_15;
  reg [32-1:0] _tmp_data_16;
  reg _tmp_valid_16;
  wire _tmp_ready_16;
  assign _tmp_ready_16 = (_tmp_ready_18 || !_tmp_valid_18) && _tmp_valid_16;
  reg [32-1:0] _tmp_data_17;
  reg _tmp_valid_17;
  wire _tmp_ready_17;
  assign _tmp_ready_17 = (_tmp_ready_19 || !_tmp_valid_19) && _tmp_valid_17;
  reg [32-1:0] _tmp_data_18;
  reg _tmp_valid_18;
  wire _tmp_ready_18;
  assign _tmp_ready_18 = (_tmp_ready_20 || !_tmp_valid_20) && _tmp_valid_18;
  reg [32-1:0] _tmp_data_19;
  reg _tmp_valid_19;
  wire _tmp_ready_19;
  assign _tmp_ready_19 = (_tmp_ready_21 || !_tmp_valid_21) && _tmp_valid_19;
  reg [32-1:0] _tmp_data_20;
  reg _tmp_valid_20;
  wire _tmp_ready_20;
  assign _tmp_ready_20 = (_tmp_ready_22 || !_tmp_valid_22) && _tmp_valid_20;
  reg [32-1:0] _tmp_data_21;
  reg _tmp_valid_21;
  wire _tmp_ready_21;
  assign _tmp_ready_21 = (_tmp_ready_23 || !_tmp_valid_23) && _tmp_valid_21;
  reg [32-1:0] _tmp_data_22;
  reg _tmp_valid_22;
  wire _tmp_ready_22;
  assign _tmp_ready_22 = (_tmp_ready_26 || !_tmp_valid_26) && _tmp_valid_22;
  reg [32-1:0] _tmp_data_23;
  reg _tmp_valid_23;
  wire _tmp_ready_23;
  assign _tmp_ready_23 = (_tmp_ready_27 || !_tmp_valid_27) && _tmp_valid_23;
  reg [32-1:0] _tmp_data_24;
  reg _tmp_valid_24;
  wire _tmp_ready_24;
  reg [32-1:0] _tmp_data_25;
  reg _tmp_valid_25;
  wire _tmp_ready_25;
  reg [32-1:0] _tmp_data_26;
  reg _tmp_valid_26;
  wire _tmp_ready_26;
  reg [32-1:0] _tmp_data_27;
  reg _tmp_valid_27;
  wire _tmp_ready_27;
  assign dout1re = _tmp_data_24;
  assign _tmp_ready_24 = 1;
  assign dout1im = _tmp_data_25;
  assign _tmp_ready_25 = 1;
  assign dout0re = _tmp_data_26;
  assign _tmp_ready_26 = 1;
  assign dout0im = _tmp_data_27;
  assign _tmp_ready_27 = 1;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_data_0 <= 0;
      _tmp_valid_0 <= 0;
      _tmp_data_1 <= 0;
      _tmp_valid_1 <= 0;
      _tmp_data_2 <= 0;
      _tmp_valid_2 <= 0;
      _tmp_data_3 <= 0;
      _tmp_valid_3 <= 0;
      _tmp_data_4 <= 0;
      _tmp_valid_4 <= 0;
      _tmp_data_5 <= 0;
      _tmp_valid_5 <= 0;
      _tmp_data_reg_6 <= 0;
      _tmp_valid_reg_6 <= 0;
      _tmp_data_reg_7 <= 0;
      _tmp_valid_reg_7 <= 0;
      _tmp_data_reg_8 <= 0;
      _tmp_valid_reg_8 <= 0;
      _tmp_data_reg_9 <= 0;
      _tmp_valid_reg_9 <= 0;
      _tmp_data_10 <= 0;
      _tmp_valid_10 <= 0;
      _tmp_data_11 <= 0;
      _tmp_valid_11 <= 0;
      _tmp_data_12 <= 0;
      _tmp_valid_12 <= 0;
      _tmp_data_13 <= 0;
      _tmp_valid_13 <= 0;
      _tmp_data_14 <= 0;
      _tmp_valid_14 <= 0;
      _tmp_data_15 <= 0;
      _tmp_valid_15 <= 0;
      _tmp_data_16 <= 0;
      _tmp_valid_16 <= 0;
      _tmp_data_17 <= 0;
      _tmp_valid_17 <= 0;
      _tmp_data_18 <= 0;
      _tmp_valid_18 <= 0;
      _tmp_data_19 <= 0;
      _tmp_valid_19 <= 0;
      _tmp_data_20 <= 0;
      _tmp_valid_20 <= 0;
      _tmp_data_21 <= 0;
      _tmp_valid_21 <= 0;
      _tmp_data_22 <= 0;
      _tmp_valid_22 <= 0;
      _tmp_data_23 <= 0;
      _tmp_valid_23 <= 0;
      _tmp_data_24 <= 0;
      _tmp_valid_24 <= 0;
      _tmp_data_25 <= 0;
      _tmp_valid_25 <= 0;
      _tmp_data_26 <= 0;
      _tmp_valid_26 <= 0;
      _tmp_data_27 <= 0;
      _tmp_valid_27 <= 0;
    end else begin
      if((_tmp_ready_0 || !_tmp_valid_0) && 1 && 1) begin
        _tmp_data_0 <= din0re + din1re;
      end 
      if(_tmp_valid_0 && _tmp_ready_0) begin
        _tmp_valid_0 <= 0;
      end 
      if((_tmp_ready_0 || !_tmp_valid_0) && 1) begin
        _tmp_valid_0 <= 1;
      end 
      if((_tmp_ready_1 || !_tmp_valid_1) && 1 && 1) begin
        _tmp_data_1 <= din0im + din1im;
      end 
      if(_tmp_valid_1 && _tmp_ready_1) begin
        _tmp_valid_1 <= 0;
      end 
      if((_tmp_ready_1 || !_tmp_valid_1) && 1) begin
        _tmp_valid_1 <= 1;
      end 
      if((_tmp_ready_2 || !_tmp_valid_2) && 1 && 1) begin
        _tmp_data_2 <= din0re - din1re;
      end 
      if(_tmp_valid_2 && _tmp_ready_2) begin
        _tmp_valid_2 <= 0;
      end 
      if((_tmp_ready_2 || !_tmp_valid_2) && 1) begin
        _tmp_valid_2 <= 1;
      end 
      if((_tmp_ready_3 || !_tmp_valid_3) && 1 && 1) begin
        _tmp_data_3 <= din0im - din1im;
      end 
      if(_tmp_valid_3 && _tmp_ready_3) begin
        _tmp_valid_3 <= 0;
      end 
      if((_tmp_ready_3 || !_tmp_valid_3) && 1) begin
        _tmp_valid_3 <= 1;
      end 
      if((_tmp_ready_4 || !_tmp_valid_4) && 1 && 1) begin
        _tmp_data_4 <= cnstre;
      end 
      if(_tmp_valid_4 && _tmp_ready_4) begin
        _tmp_valid_4 <= 0;
      end 
      if((_tmp_ready_4 || !_tmp_valid_4) && 1) begin
        _tmp_valid_4 <= 1;
      end 
      if((_tmp_ready_5 || !_tmp_valid_5) && 1 && 1) begin
        _tmp_data_5 <= cnstim;
      end 
      if(_tmp_valid_5 && _tmp_ready_5) begin
        _tmp_valid_5 <= 0;
      end 
      if((_tmp_ready_5 || !_tmp_valid_5) && 1) begin
        _tmp_valid_5 <= 1;
      end 
      if(_tmp_ready_6 || !_tmp_valid_6) begin
        _tmp_data_reg_6 <= _tmp_odata_6;
      end 
      if(_tmp_ready_6 || !_tmp_valid_6) begin
        _tmp_valid_reg_6 <= _tmp_ovalid_6;
      end 
      if(_tmp_ready_7 || !_tmp_valid_7) begin
        _tmp_data_reg_7 <= _tmp_odata_7;
      end 
      if(_tmp_ready_7 || !_tmp_valid_7) begin
        _tmp_valid_reg_7 <= _tmp_ovalid_7;
      end 
      if(_tmp_ready_8 || !_tmp_valid_8) begin
        _tmp_data_reg_8 <= _tmp_odata_8;
      end 
      if(_tmp_ready_8 || !_tmp_valid_8) begin
        _tmp_valid_reg_8 <= _tmp_ovalid_8;
      end 
      if(_tmp_ready_9 || !_tmp_valid_9) begin
        _tmp_data_reg_9 <= _tmp_odata_9;
      end 
      if(_tmp_ready_9 || !_tmp_valid_9) begin
        _tmp_valid_reg_9 <= _tmp_ovalid_9;
      end 
      if((_tmp_ready_10 || !_tmp_valid_10) && _tmp_ready_0 && _tmp_valid_0) begin
        _tmp_data_10 <= _tmp_data_0;
      end 
      if(_tmp_valid_10 && _tmp_ready_10) begin
        _tmp_valid_10 <= 0;
      end 
      if((_tmp_ready_10 || !_tmp_valid_10) && _tmp_ready_0) begin
        _tmp_valid_10 <= _tmp_valid_0;
      end 
      if((_tmp_ready_11 || !_tmp_valid_11) && _tmp_ready_1 && _tmp_valid_1) begin
        _tmp_data_11 <= _tmp_data_1;
      end 
      if(_tmp_valid_11 && _tmp_ready_11) begin
        _tmp_valid_11 <= 0;
      end 
      if((_tmp_ready_11 || !_tmp_valid_11) && _tmp_ready_1) begin
        _tmp_valid_11 <= _tmp_valid_1;
      end 
      if((_tmp_ready_12 || !_tmp_valid_12) && _tmp_ready_10 && _tmp_valid_10) begin
        _tmp_data_12 <= _tmp_data_10;
      end 
      if(_tmp_valid_12 && _tmp_ready_12) begin
        _tmp_valid_12 <= 0;
      end 
      if((_tmp_ready_12 || !_tmp_valid_12) && _tmp_ready_10) begin
        _tmp_valid_12 <= _tmp_valid_10;
      end 
      if((_tmp_ready_13 || !_tmp_valid_13) && _tmp_ready_11 && _tmp_valid_11) begin
        _tmp_data_13 <= _tmp_data_11;
      end 
      if(_tmp_valid_13 && _tmp_ready_13) begin
        _tmp_valid_13 <= 0;
      end 
      if((_tmp_ready_13 || !_tmp_valid_13) && _tmp_ready_11) begin
        _tmp_valid_13 <= _tmp_valid_11;
      end 
      if((_tmp_ready_14 || !_tmp_valid_14) && _tmp_ready_12 && _tmp_valid_12) begin
        _tmp_data_14 <= _tmp_data_12;
      end 
      if(_tmp_valid_14 && _tmp_ready_14) begin
        _tmp_valid_14 <= 0;
      end 
      if((_tmp_ready_14 || !_tmp_valid_14) && _tmp_ready_12) begin
        _tmp_valid_14 <= _tmp_valid_12;
      end 
      if((_tmp_ready_15 || !_tmp_valid_15) && _tmp_ready_13 && _tmp_valid_13) begin
        _tmp_data_15 <= _tmp_data_13;
      end 
      if(_tmp_valid_15 && _tmp_ready_15) begin
        _tmp_valid_15 <= 0;
      end 
      if((_tmp_ready_15 || !_tmp_valid_15) && _tmp_ready_13) begin
        _tmp_valid_15 <= _tmp_valid_13;
      end 
      if((_tmp_ready_16 || !_tmp_valid_16) && _tmp_ready_14 && _tmp_valid_14) begin
        _tmp_data_16 <= _tmp_data_14;
      end 
      if(_tmp_valid_16 && _tmp_ready_16) begin
        _tmp_valid_16 <= 0;
      end 
      if((_tmp_ready_16 || !_tmp_valid_16) && _tmp_ready_14) begin
        _tmp_valid_16 <= _tmp_valid_14;
      end 
      if((_tmp_ready_17 || !_tmp_valid_17) && _tmp_ready_15 && _tmp_valid_15) begin
        _tmp_data_17 <= _tmp_data_15;
      end 
      if(_tmp_valid_17 && _tmp_ready_17) begin
        _tmp_valid_17 <= 0;
      end 
      if((_tmp_ready_17 || !_tmp_valid_17) && _tmp_ready_15) begin
        _tmp_valid_17 <= _tmp_valid_15;
      end 
      if((_tmp_ready_18 || !_tmp_valid_18) && _tmp_ready_16 && _tmp_valid_16) begin
        _tmp_data_18 <= _tmp_data_16;
      end 
      if(_tmp_valid_18 && _tmp_ready_18) begin
        _tmp_valid_18 <= 0;
      end 
      if((_tmp_ready_18 || !_tmp_valid_18) && _tmp_ready_16) begin
        _tmp_valid_18 <= _tmp_valid_16;
      end 
      if((_tmp_ready_19 || !_tmp_valid_19) && _tmp_ready_17 && _tmp_valid_17) begin
        _tmp_data_19 <= _tmp_data_17;
      end 
      if(_tmp_valid_19 && _tmp_ready_19) begin
        _tmp_valid_19 <= 0;
      end 
      if((_tmp_ready_19 || !_tmp_valid_19) && _tmp_ready_17) begin
        _tmp_valid_19 <= _tmp_valid_17;
      end 
      if((_tmp_ready_20 || !_tmp_valid_20) && _tmp_ready_18 && _tmp_valid_18) begin
        _tmp_data_20 <= _tmp_data_18;
      end 
      if(_tmp_valid_20 && _tmp_ready_20) begin
        _tmp_valid_20 <= 0;
      end 
      if((_tmp_ready_20 || !_tmp_valid_20) && _tmp_ready_18) begin
        _tmp_valid_20 <= _tmp_valid_18;
      end 
      if((_tmp_ready_21 || !_tmp_valid_21) && _tmp_ready_19 && _tmp_valid_19) begin
        _tmp_data_21 <= _tmp_data_19;
      end 
      if(_tmp_valid_21 && _tmp_ready_21) begin
        _tmp_valid_21 <= 0;
      end 
      if((_tmp_ready_21 || !_tmp_valid_21) && _tmp_ready_19) begin
        _tmp_valid_21 <= _tmp_valid_19;
      end 
      if((_tmp_ready_22 || !_tmp_valid_22) && _tmp_ready_20 && _tmp_valid_20) begin
        _tmp_data_22 <= _tmp_data_20;
      end 
      if(_tmp_valid_22 && _tmp_ready_22) begin
        _tmp_valid_22 <= 0;
      end 
      if((_tmp_ready_22 || !_tmp_valid_22) && _tmp_ready_20) begin
        _tmp_valid_22 <= _tmp_valid_20;
      end 
      if((_tmp_ready_23 || !_tmp_valid_23) && _tmp_ready_21 && _tmp_valid_21) begin
        _tmp_data_23 <= _tmp_data_21;
      end 
      if(_tmp_valid_23 && _tmp_ready_23) begin
        _tmp_valid_23 <= 0;
      end 
      if((_tmp_ready_23 || !_tmp_valid_23) && _tmp_ready_21) begin
        _tmp_valid_23 <= _tmp_valid_21;
      end 
      if((_tmp_ready_24 || !_tmp_valid_24) && (_tmp_ready_6 && _tmp_ready_7) && (_tmp_valid_6 && _tmp_valid_7)) begin
        _tmp_data_24 <= _tmp_data_6 - _tmp_data_7;
      end 
      if(_tmp_valid_24 && _tmp_ready_24) begin
        _tmp_valid_24 <= 0;
      end 
      if((_tmp_ready_24 || !_tmp_valid_24) && (_tmp_ready_6 && _tmp_ready_7)) begin
        _tmp_valid_24 <= _tmp_valid_6 && _tmp_valid_7;
      end 
      if((_tmp_ready_25 || !_tmp_valid_25) && (_tmp_ready_8 && _tmp_ready_9) && (_tmp_valid_8 && _tmp_valid_9)) begin
        _tmp_data_25 <= _tmp_data_8 + _tmp_data_9;
      end 
      if(_tmp_valid_25 && _tmp_ready_25) begin
        _tmp_valid_25 <= 0;
      end 
      if((_tmp_ready_25 || !_tmp_valid_25) && (_tmp_ready_8 && _tmp_ready_9)) begin
        _tmp_valid_25 <= _tmp_valid_8 && _tmp_valid_9;
      end 
      if((_tmp_ready_26 || !_tmp_valid_26) && _tmp_ready_22 && _tmp_valid_22) begin
        _tmp_data_26 <= _tmp_data_22;
      end 
      if(_tmp_valid_26 && _tmp_ready_26) begin
        _tmp_valid_26 <= 0;
      end 
      if((_tmp_ready_26 || !_tmp_valid_26) && _tmp_ready_22) begin
        _tmp_valid_26 <= _tmp_valid_22;
      end 
      if((_tmp_ready_27 || !_tmp_valid_27) && _tmp_ready_23 && _tmp_valid_23) begin
        _tmp_data_27 <= _tmp_data_23;
      end 
      if(_tmp_valid_27 && _tmp_ready_27) begin
        _tmp_valid_27 <= 0;
      end 
      if((_tmp_ready_27 || !_tmp_valid_27) && _tmp_ready_23) begin
        _tmp_valid_27 <= _tmp_valid_23;
      end 
    end
  end


endmodule



module multiplier_0
(
  input CLK,
  input RST,
  input update,
  input enable,
  output valid,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg valid_reg0;
  reg valid_reg1;
  reg valid_reg2;
  reg valid_reg3;
  reg valid_reg4;
  reg valid_reg5;
  assign valid = valid_reg5;

  always @(posedge CLK) begin
    if(RST) begin
      valid_reg0 <= 0;
      valid_reg1 <= 0;
      valid_reg2 <= 0;
      valid_reg3 <= 0;
      valid_reg4 <= 0;
      valid_reg5 <= 0;
    end else begin
      if(update) begin
        valid_reg0 <= enable;
        valid_reg1 <= valid_reg0;
        valid_reg2 <= valid_reg1;
        valid_reg3 <= valid_reg2;
        valid_reg4 <= valid_reg3;
        valid_reg5 <= valid_reg4;
      end 
    end
  end


  multiplier_core_0
  mult
  (
    .CLK(CLK),
    .update(update),
    .a(a),
    .b(b),
    .c(c)
  );


endmodule



module multiplier_core_0
(
  input CLK,
  input update,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg [32-1:0] _a;
  reg [32-1:0] _b;
  reg signed [64-1:0] _tmpval0;
  reg signed [64-1:0] _tmpval1;
  reg signed [64-1:0] _tmpval2;
  reg signed [64-1:0] _tmpval3;
  reg signed [64-1:0] _tmpval4;
  wire signed [64-1:0] rslt;
  assign rslt = $signed({ 1'd0, _a }) * $signed({ 1'd0, _b });
  assign c = _tmpval4;

  always @(posedge CLK) begin
    if(update) begin
      _a <= a;
      _b <= b;
      _tmpval0 <= rslt;
      _tmpval1 <= _tmpval0;
      _tmpval2 <= _tmpval1;
      _tmpval3 <= _tmpval2;
      _tmpval4 <= _tmpval3;
    end 
  end


endmodule



module multiplier_1
(
  input CLK,
  input RST,
  input update,
  input enable,
  output valid,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg valid_reg0;
  reg valid_reg1;
  reg valid_reg2;
  reg valid_reg3;
  reg valid_reg4;
  reg valid_reg5;
  assign valid = valid_reg5;

  always @(posedge CLK) begin
    if(RST) begin
      valid_reg0 <= 0;
      valid_reg1 <= 0;
      valid_reg2 <= 0;
      valid_reg3 <= 0;
      valid_reg4 <= 0;
      valid_reg5 <= 0;
    end else begin
      if(update) begin
        valid_reg0 <= enable;
        valid_reg1 <= valid_reg0;
        valid_reg2 <= valid_reg1;
        valid_reg3 <= valid_reg2;
        valid_reg4 <= valid_reg3;
        valid_reg5 <= valid_reg4;
      end 
    end
  end


  multiplier_core_1
  mult
  (
    .CLK(CLK),
    .update(update),
    .a(a),
    .b(b),
    .c(c)
  );


endmodule



module multiplier_core_1
(
  input CLK,
  input update,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg [32-1:0] _a;
  reg [32-1:0] _b;
  reg signed [64-1:0] _tmpval0;
  reg signed [64-1:0] _tmpval1;
  reg signed [64-1:0] _tmpval2;
  reg signed [64-1:0] _tmpval3;
  reg signed [64-1:0] _tmpval4;
  wire signed [64-1:0] rslt;
  assign rslt = $signed({ 1'd0, _a }) * $signed({ 1'd0, _b });
  assign c = _tmpval4;

  always @(posedge CLK) begin
    if(update) begin
      _a <= a;
      _b <= b;
      _tmpval0 <= rslt;
      _tmpval1 <= _tmpval0;
      _tmpval2 <= _tmpval1;
      _tmpval3 <= _tmpval2;
      _tmpval4 <= _tmpval3;
    end 
  end


endmodule



module multiplier_2
(
  input CLK,
  input RST,
  input update,
  input enable,
  output valid,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg valid_reg0;
  reg valid_reg1;
  reg valid_reg2;
  reg valid_reg3;
  reg valid_reg4;
  reg valid_reg5;
  assign valid = valid_reg5;

  always @(posedge CLK) begin
    if(RST) begin
      valid_reg0 <= 0;
      valid_reg1 <= 0;
      valid_reg2 <= 0;
      valid_reg3 <= 0;
      valid_reg4 <= 0;
      valid_reg5 <= 0;
    end else begin
      if(update) begin
        valid_reg0 <= enable;
        valid_reg1 <= valid_reg0;
        valid_reg2 <= valid_reg1;
        valid_reg3 <= valid_reg2;
        valid_reg4 <= valid_reg3;
        valid_reg5 <= valid_reg4;
      end 
    end
  end


  multiplier_core_2
  mult
  (
    .CLK(CLK),
    .update(update),
    .a(a),
    .b(b),
    .c(c)
  );


endmodule



module multiplier_core_2
(
  input CLK,
  input update,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg [32-1:0] _a;
  reg [32-1:0] _b;
  reg signed [64-1:0] _tmpval0;
  reg signed [64-1:0] _tmpval1;
  reg signed [64-1:0] _tmpval2;
  reg signed [64-1:0] _tmpval3;
  reg signed [64-1:0] _tmpval4;
  wire signed [64-1:0] rslt;
  assign rslt = $signed({ 1'd0, _a }) * $signed({ 1'd0, _b });
  assign c = _tmpval4;

  always @(posedge CLK) begin
    if(update) begin
      _a <= a;
      _b <= b;
      _tmpval0 <= rslt;
      _tmpval1 <= _tmpval0;
      _tmpval2 <= _tmpval1;
      _tmpval3 <= _tmpval2;
      _tmpval4 <= _tmpval3;
    end 
  end


endmodule



module multiplier_3
(
  input CLK,
  input RST,
  input update,
  input enable,
  output valid,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg valid_reg0;
  reg valid_reg1;
  reg valid_reg2;
  reg valid_reg3;
  reg valid_reg4;
  reg valid_reg5;
  assign valid = valid_reg5;

  always @(posedge CLK) begin
    if(RST) begin
      valid_reg0 <= 0;
      valid_reg1 <= 0;
      valid_reg2 <= 0;
      valid_reg3 <= 0;
      valid_reg4 <= 0;
      valid_reg5 <= 0;
    end else begin
      if(update) begin
        valid_reg0 <= enable;
        valid_reg1 <= valid_reg0;
        valid_reg2 <= valid_reg1;
        valid_reg3 <= valid_reg2;
        valid_reg4 <= valid_reg3;
        valid_reg5 <= valid_reg4;
      end 
    end
  end


  multiplier_core_3
  mult
  (
    .CLK(CLK),
    .update(update),
    .a(a),
    .b(b),
    .c(c)
  );


endmodule



module multiplier_core_3
(
  input CLK,
  input update,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg [32-1:0] _a;
  reg [32-1:0] _b;
  reg signed [64-1:0] _tmpval0;
  reg signed [64-1:0] _tmpval1;
  reg signed [64-1:0] _tmpval2;
  reg signed [64-1:0] _tmpval3;
  reg signed [64-1:0] _tmpval4;
  wire signed [64-1:0] rslt;
  assign rslt = $signed({ 1'd0, _a }) * $signed({ 1'd0, _b });
  assign c = _tmpval4;

  always @(posedge CLK) begin
    if(update) begin
      _a <= a;
      _b <= b;
      _tmpval0 <= rslt;
      _tmpval1 <= _tmpval0;
      _tmpval2 <= _tmpval1;
      _tmpval3 <= _tmpval2;
      _tmpval4 <= _tmpval3;
    end 
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = dataflow_radix2.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import dataflow_movmin

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  reg signed [32-1:0] xdata;
  reg xvalid;
  wire xready;
  wire signed [32-1:0] ydata;
  wire yvalid;
  reg yready;

  movmin
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .xdata(xdata),
    .xvalid(xvalid),
    .xready(xready),
    .ydata(ydata),
    .yvalid(yvalid),
    .yready(yready)
  );

  reg reset_done;

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    reset_done = 0;
    xdata = 0;
    xvalid = 0;
    yready = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    reset_done = 1;
    @(posedge CLK);
    #1;
    #10000;
    $finish;
  end

  reg [32-1:0] xfsm;
  localparam xfsm_init = 0;
  reg [32-1:0] _tmp_0;
  localparam xfsm_1 = 1;
  localparam xfsm_2 = 2;
  localparam xfsm_3 = 3;
  localparam xfsm_4 = 4;
  localparam xfsm_5 = 5;
  localparam xfsm_6 = 6;
  localparam xfsm_7 = 7;
  localparam xfsm_8 = 8;
  localparam xfsm_9 = 9;
  localparam xfsm_10 = 10;
  localparam xfsm_11 = 11;
  localparam xfsm_12 = 12;
  localparam xfsm_13 = 13;
  localparam xfsm_14 = 14;
  localparam xfsm_15 = 15;
  localparam xfsm_16 = 16;
  localparam xfsm_17 = 17;
  localparam xfsm_18 = 18;
  localparam xfsm_19 = 19;
  localparam xfsm_20 = 20;
  localparam xfsm_21 = 21;
  localparam xfsm_22 = 22;
  localparam xfsm_23 = 23;
  localparam xfsm_24 = 24;

  always @(posedge CLK) begin
    if(RST) begin
      xfsm <= xfsm_init;
      _tmp_0 <= 0;
    end else begin
      case(xfsm)
        xfsm_init: begin
          xvalid <= 0;
          if(reset_done) begin
            xfsm <= xfsm_1;
          end 
        end
        xfsm_1: begin
          xfsm <= xfsm_2;
        end
        xfsm_2: begin
          xfsm <= xfsm_3;
        end
        xfsm_3: begin
          xfsm <= xfsm_4;
        end
        xfsm_4: begin
          xfsm <= xfsm_5;
        end
        xfsm_5: begin
          xfsm <= xfsm_6;
        end
        xfsm_6: begin
          xfsm <= xfsm_7;
        end
        xfsm_7: begin
          xfsm <= xfsm_8;
        end
        xfsm_8: begin
          xfsm <= xfsm_9;
        end
        xfsm_9: begin
          xfsm <= xfsm_10;
        end
        xfsm_10: begin
          xfsm <= xfsm_11;
        end
        xfsm_11: begin
          xvalid <= 1;
          xfsm <= xfsm_12;
        end
        xfsm_12: begin
          if(xready) begin
            xdata <= xdata + 1;
          end 
          if(xready) begin
            _tmp_0 <= _tmp_0 + 1;
          end 
          if((_tmp_0 == 15) && xready) begin
            xfsm <= xfsm_13;
          end 
        end
        xfsm_13: begin
          xvalid <= 0;
          xfsm <= xfsm_14;
        end
        xfsm_14: begin
          xfsm <= xfsm_15;
        end
        xfsm_15: begin
          xfsm <= xfsm_16;
        end
        xfsm_16: begin
          xfsm <= xfsm_17;
        end
        xfsm_17: begin
          xfsm <= xfsm_18;
        end
        xfsm_18: begin
          xfsm <= xfsm_19;
        end
        xfsm_19: begin
          xfsm <= xfsm_20;
        end
        xfsm_20: begin
          xfsm <= xfsm_21;
        end
        xfsm_21: begin
          xfsm <= xfsm_22;
        end
        xfsm_22: begin
          xfsm <= xfsm_23;
        end
        xfsm_23: begin
          xvalid <= 1;
          if(xready) begin
            xdata <= xdata + 1;
          end 
          if(xready) begin
            _tmp_0 <= _tmp_0 + 1;
          end 
          if((_tmp_0 == 30) && xready) begin
            xfsm <= xfsm_24;
          end 
        end
        xfsm_24: begin
          xvalid <= 0;
        end
      endcase
    end
  end

  reg [32-1:0] yfsm;
  localparam yfsm_init = 0;
  localparam yfsm_1 = 1;
  localparam yfsm_2 = 2;
  localparam yfsm_3 = 3;
  localparam yfsm_4 = 4;
  localparam yfsm_5 = 5;
  localparam yfsm_6 = 6;
  localparam yfsm_7 = 7;
  localparam yfsm_8 = 8;

  always @(posedge CLK) begin
    if(RST) begin
      yfsm <= yfsm_init;
    end else begin
      case(yfsm)
        yfsm_init: begin
          yready <= 0;
          if(reset_done) begin
            yfsm <= yfsm_1;
          end 
        end
        yfsm_1: begin
          yfsm <= yfsm_2;
        end
        yfsm_2: begin
          if(yvalid) begin
            yready <= 1;
          end 
          if(yvalid) begin
            yfsm <= yfsm_3;
          end 
        end
        yfsm_3: begin
          yready <= 0;
          yfsm <= yfsm_4;
        end
        yfsm_4: begin
          yready <= 0;
          yfsm <= yfsm_5;
        end
        yfsm_5: begin
          yready <= 0;
          yfsm <= yfsm_6;
        end
        yfsm_6: begin
          yready <= 0;
          yfsm <= yfsm_7;
        end
        yfsm_7: begin
          yready <= 0;
          yfsm <= yfsm_8;
        end
        yfsm_8: begin
          yfsm <= yfsm_2;
        end
      endcase
    end
  end


  always @(posedge CLK) begin
    if(reset_done) begin
      if(xvalid && xready) begin
        $display("xdata=%d", xdata);
      end 
      if(yvalid && yready) begin
        $display("ydata=%d", ydata);
      end 
    end 
  end


endmodule



module movmin
(
  input CLK,
  input RST,
  input signed [32-1:0] xdata,
  input xvalid,
  output xready,
  output signed [32-1:0] ydata,
  output yvalid,
  input yready
);

  reg signed [32-1:0] _tmp_data_0;
  reg signed [32-1:0] _tmp_data_1;
  reg signed [32-1:0] _tmp_data_2;
  reg signed [32-1:0] _tmp_data_3;
  reg signed [32-1:0] _tmp_data_4;
  reg signed [32-1:0] _tmp_data_5;
  reg signed [32-1:0] _tmp_data_6;
  reg [1-1:0] _tmp_data_7;
  reg _tmp_valid_7;
  wire _tmp_ready_7;
  assign xready = (_tmp_ready_7 || !_tmp_valid_7) && (xvalid && xvalid) && ((_tmp_ready_7 || !_tmp_valid_7) && (xvalid && xvalid)) && ((_tmp_ready_8 || !_tmp_valid_8) && (xvalid && xvalid)) && ((_tmp_ready_8 || !_tmp_valid_8) && (xvalid && xvalid)) && ((_tmp_ready_9 || !_tmp_valid_9) && (xvalid && xvalid)) && ((_tmp_ready_9 || !_tmp_valid_9) && (xvalid && xvalid)) && ((_tmp_ready_10 || !_tmp_valid_10) && (xvalid && xvalid)) && ((_tmp_ready_10 || !_tmp_valid_10) && (xvalid && xvalid)) && ((_tmp_ready_11 || !_tmp_valid_11) && xvalid) && ((_tmp_ready_12 || !_tmp_valid_12) && xvalid) && ((_tmp_ready_13 || !_tmp_valid_13) && xvalid) && ((_tmp_ready_14 || !_tmp_valid_14) && xvalid) && ((_tmp_ready_15 || !_tmp_valid_15) && xvalid) && ((_tmp_ready_16 || !_tmp_valid_16) && xvalid) && ((_tmp_ready_17 || !_tmp_valid_17) && xvalid) && ((_tmp_ready_18 || !_tmp_valid_18) && xvalid);
  assign _tmp_ready_7 = (_tmp_ready_19 || !_tmp_valid_19) && (_tmp_valid_7 && _tmp_valid_11 && _tmp_valid_12);
  reg [1-1:0] _tmp_data_8;
  reg _tmp_valid_8;
  wire _tmp_ready_8;
  assign _tmp_ready_8 = (_tmp_ready_20 || !_tmp_valid_20) && (_tmp_valid_8 && _tmp_valid_13 && _tmp_valid_14);
  reg [1-1:0] _tmp_data_9;
  reg _tmp_valid_9;
  wire _tmp_ready_9;
  assign _tmp_ready_9 = (_tmp_ready_21 || !_tmp_valid_21) && (_tmp_valid_9 && _tmp_valid_15 && _tmp_valid_16);
  reg [1-1:0] _tmp_data_10;
  reg _tmp_valid_10;
  wire _tmp_ready_10;
  assign _tmp_ready_10 = (_tmp_ready_22 || !_tmp_valid_22) && (_tmp_valid_10 && _tmp_valid_17 && _tmp_valid_18);
  reg signed [32-1:0] _tmp_data_11;
  reg _tmp_valid_11;
  wire _tmp_ready_11;
  assign _tmp_ready_11 = (_tmp_ready_19 || !_tmp_valid_19) && (_tmp_valid_7 && _tmp_valid_11 && _tmp_valid_12);
  reg signed [32-1:0] _tmp_data_12;
  reg _tmp_valid_12;
  wire _tmp_ready_12;
  assign _tmp_ready_12 = (_tmp_ready_19 || !_tmp_valid_19) && (_tmp_valid_7 && _tmp_valid_11 && _tmp_valid_12);
  reg signed [32-1:0] _tmp_data_13;
  reg _tmp_valid_13;
  wire _tmp_ready_13;
  assign _tmp_ready_13 = (_tmp_ready_20 || !_tmp_valid_20) && (_tmp_valid_8 && _tmp_valid_13 && _tmp_valid_14);
  reg signed [32-1:0] _tmp_data_14;
  reg _tmp_valid_14;
  wire _tmp_ready_14;
  assign _tmp_ready_14 = (_tmp_ready_20 || !_tmp_valid_20) && (_tmp_valid_8 && _tmp_valid_13 && _tmp_valid_14);
  reg signed [32-1:0] _tmp_data_15;
  reg _tmp_valid_15;
  wire _tmp_ready_15;
  assign _tmp_ready_15 = (_tmp_ready_21 || !_tmp_valid_21) && (_tmp_valid_9 && _tmp_valid_15 && _tmp_valid_16);
  reg signed [32-1:0] _tmp_data_16;
  reg _tmp_valid_16;
  wire _tmp_ready_16;
  assign _tmp_ready_16 = (_tmp_ready_21 || !_tmp_valid_21) && (_tmp_valid_9 && _tmp_valid_15 && _tmp_valid_16);
  reg signed [32-1:0] _tmp_data_17;
  reg _tmp_valid_17;
  wire _tmp_ready_17;
  assign _tmp_ready_17 = (_tmp_ready_22 || !_tmp_valid_22) && (_tmp_valid_10 && _tmp_valid_17 && _tmp_valid_18);
  reg signed [32-1:0] _tmp_data_18;
  reg _tmp_valid_18;
  wire _tmp_ready_18;
  assign _tmp_ready_18 = (_tmp_ready_22 || !_tmp_valid_22) && (_tmp_valid_10 && _tmp_valid_17 && _tmp_valid_18);
  reg signed [32-1:0] _tmp_data_19;
  reg _tmp_valid_19;
  wire _tmp_ready_19;
  assign _tmp_ready_19 = (_tmp_ready_23 || !_tmp_valid_23) && (_tmp_valid_19 && _tmp_valid_20) && ((_tmp_ready_25 || !_tmp_valid_25) && _tmp_valid_19);
  reg signed [32-1:0] _tmp_data_20;
  reg _tmp_valid_20;
  wire _tmp_ready_20;
  assign _tmp_ready_20 = (_tmp_ready_23 || !_tmp_valid_23) && (_tmp_valid_19 && _tmp_valid_20) && ((_tmp_ready_26 || !_tmp_valid_26) && _tmp_valid_20);
  reg signed [32-1:0] _tmp_data_21;
  reg _tmp_valid_21;
  wire _tmp_ready_21;
  assign _tmp_ready_21 = (_tmp_ready_24 || !_tmp_valid_24) && (_tmp_valid_21 && _tmp_valid_22) && ((_tmp_ready_27 || !_tmp_valid_27) && _tmp_valid_21);
  reg signed [32-1:0] _tmp_data_22;
  reg _tmp_valid_22;
  wire _tmp_ready_22;
  assign _tmp_ready_22 = (_tmp_ready_24 || !_tmp_valid_24) && (_tmp_valid_21 && _tmp_valid_22) && ((_tmp_ready_28 || !_tmp_valid_28) && _tmp_valid_22);
  reg [1-1:0] _tmp_data_23;
  reg _tmp_valid_23;
  wire _tmp_ready_23;
  assign _tmp_ready_23 = (_tmp_ready_29 || !_tmp_valid_29) && (_tmp_valid_23 && _tmp_valid_25 && _tmp_valid_26);
  reg [1-1:0] _tmp_data_24;
  reg _tmp_valid_24;
  wire _tmp_ready_24;
  assign _tmp_ready_24 = (_tmp_ready_30 || !_tmp_valid_30) && (_tmp_valid_24 && _tmp_valid_27 && _tmp_valid_28);
  reg signed [32-1:0] _tmp_data_25;
  reg _tmp_valid_25;
  wire _tmp_ready_25;
  assign _tmp_ready_25 = (_tmp_ready_29 || !_tmp_valid_29) && (_tmp_valid_23 && _tmp_valid_25 && _tmp_valid_26);
  reg signed [32-1:0] _tmp_data_26;
  reg _tmp_valid_26;
  wire _tmp_ready_26;
  assign _tmp_ready_26 = (_tmp_ready_29 || !_tmp_valid_29) && (_tmp_valid_23 && _tmp_valid_25 && _tmp_valid_26);
  reg signed [32-1:0] _tmp_data_27;
  reg _tmp_valid_27;
  wire _tmp_ready_27;
  assign _tmp_ready_27 = (_tmp_ready_30 || !_tmp_valid_30) && (_tmp_valid_24 && _tmp_valid_27 && _tmp_valid_28);
  reg signed [32-1:0] _tmp_data_28;
  reg _tmp_valid_28;
  wire _tmp_ready_28;
  assign _tmp_ready_28 = (_tmp_ready_30 || !_tmp_valid_30) && (_tmp_valid_24 && _tmp_valid_27 && _tmp_valid_28);
  reg signed [32-1:0] _tmp_data_29;
  reg _tmp_valid_29;
  wire _tmp_ready_29;
  assign _tmp_ready_29 = (_tmp_ready_31 || !_tmp_valid_31) && (_tmp_valid_29 && _tmp_valid_30) && ((_tmp_ready_32 || !_tmp_valid_32) && _tmp_valid_29);
  reg signed [32-1:0] _tmp_data_30;
  reg _tmp_valid_30;
  wire _tmp_ready_30;
  assign _tmp_ready_30 = (_tmp_ready_31 || !_tmp_valid_31) && (_tmp_valid_29 && _tmp_valid_30) && ((_tmp_ready_33 || !_tmp_valid_33) && _tmp_valid_30);
  reg [1-1:0] _tmp_data_31;
  reg _tmp_valid_31;
  wire _tmp_ready_31;
  assign _tmp_ready_31 = (_tmp_ready_34 || !_tmp_valid_34) && (_tmp_valid_31 && _tmp_valid_32 && _tmp_valid_33);
  reg signed [32-1:0] _tmp_data_32;
  reg _tmp_valid_32;
  wire _tmp_ready_32;
  assign _tmp_ready_32 = (_tmp_ready_34 || !_tmp_valid_34) && (_tmp_valid_31 && _tmp_valid_32 && _tmp_valid_33);
  reg signed [32-1:0] _tmp_data_33;
  reg _tmp_valid_33;
  wire _tmp_ready_33;
  assign _tmp_ready_33 = (_tmp_ready_34 || !_tmp_valid_34) && (_tmp_valid_31 && _tmp_valid_32 && _tmp_valid_33);
  reg signed [32-1:0] _tmp_data_34;
  reg _tmp_valid_34;
  wire _tmp_ready_34;
  assign ydata = _tmp_data_34;
  assign yvalid = _tmp_valid_34;
  assign _tmp_ready_34 = yready;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_data_0 <= 0;
      _tmp_data_1 <= 0;
      _tmp_data_2 <= 0;
      _tmp_data_3 <= 0;
      _tmp_data_4 <= 0;
      _tmp_data_5 <= 0;
      _tmp_data_6 <= 0;
      _tmp_data_7 <= 0;
      _tmp_valid_7 <= 0;
      _tmp_data_8 <= 0;
      _tmp_valid_8 <= 0;
      _tmp_data_9 <= 0;
      _tmp_valid_9 <= 0;
      _tmp_data_10 <= 0;
      _tmp_valid_10 <= 0;
      _tmp_data_11 <= 0;
      _tmp_valid_11 <= 0;
      _tmp_data_12 <= 0;
      _tmp_valid_12 <= 0;
      _tmp_data_13 <= 0;
      _tmp_valid_13 <= 0;
      _tmp_data_14 <= 0;
      _tmp_valid_14 <= 0;
      _tmp_data_15 <= 0;
      _tmp_valid_15 <= 0;
      _tmp_data_16 <= 0;
      _tmp_valid_16 <= 0;
      _tmp_data_17 <= 0;
      _tmp_valid_17 <= 0;
      _tmp_data_18 <= 0;
      _tmp_valid_18 <= 0;
      _tmp_data_19 <= 0;
      _tmp_valid_19 <= 0;
      _tmp_data_20 <= 0;
      _tmp_valid_20 <= 0;
      _tmp_data_21 <= 0;
      _tmp_valid_21 <= 0;
      _tmp_data_22 <= 0;
      _tmp_valid_22 <= 0;
      _tmp_data_23 <= 0;
      _tmp_valid_23 <= 0;
      _tmp_data_24 <= 0;
      _tmp_valid_24 <= 0;
      _tmp_data_25 <= 0;
      _tmp_valid_25 <= 0;
      _tmp_data_26 <= 0;
      _tmp_valid_26 <= 0;
      _tmp_data_27 <= 0;
      _tmp_valid_27 <= 0;
      _tmp_data_28 <= 0;
      _tmp_valid_28 <= 0;
      _tmp_data_29 <= 0;
      _tmp_valid_29 <= 0;
      _tmp_data_30 <= 0;
      _tmp_valid_30 <= 0;
      _tmp_data_31 <= 0;
      _tmp_valid_31 <= 0;
      _tmp_data_32 <= 0;
      _tmp_valid_32 <= 0;
      _tmp_data_33 <= 0;
      _tmp_valid_33 <= 0;
      _tmp_data_34 <= 0;
      _tmp_valid_34 <= 0;
    end else begin
      if(xvalid && xready) begin
        _tmp_data_0 <= xdata;
      end 
      if(xvalid && xready) begin
        _tmp_data_1 <= _tmp_data_0;
      end 
      if(xvalid && xready) begin
        _tmp_data_2 <= _tmp_data_1;
      end 
      if(xvalid && xready) begin
        _tmp_data_3 <= _tmp_data_2;
      end 
      if(xvalid && xready) begin
        _tmp_data_4 <= _tmp_data_3;
      end 
      if(xvalid && xready) begin
        _tmp_data_5 <= _tmp_data_4;
      end 
      if(xvalid && xready) begin
        _tmp_data_6 <= _tmp_data_5;
      end 
      if((_tmp_ready_7 || !_tmp_valid_7) && (xready && xready) && (xvalid && xvalid)) begin
        _tmp_data_7 <= xdata < _tmp_data_0;
      end 
      if(_tmp_valid_7 && _tmp_ready_7) begin
        _tmp_valid_7 <= 0;
      end 
      if((_tmp_ready_7 || !_tmp_valid_7) && (xready && xready)) begin
        _tmp_valid_7 <= xvalid && xvalid;
      end 
      if((_tmp_ready_8 || !_tmp_valid_8) && (xready && xready) && (xvalid && xvalid)) begin
        _tmp_data_8 <= _tmp_data_1 < _tmp_data_2;
      end 
      if(_tmp_valid_8 && _tmp_ready_8) begin
        _tmp_valid_8 <= 0;
      end 
      if((_tmp_ready_8 || !_tmp_valid_8) && (xready && xready)) begin
        _tmp_valid_8 <= xvalid && xvalid;
      end 
      if((_tmp_ready_9 || !_tmp_valid_9) && (xready && xready) && (xvalid && xvalid)) begin
        _tmp_data_9 <= _tmp_data_3 < _tmp_data_4;
      end 
      if(_tmp_valid_9 && _tmp_ready_9) begin
        _tmp_valid_9 <= 0;
      end 
      if((_tmp_ready_9 || !_tmp_valid_9) && (xready && xready)) begin
        _tmp_valid_9 <= xvalid && xvalid;
      end 
      if((_tmp_ready_10 || !_tmp_valid_10) && (xready && xready) && (xvalid && xvalid)) begin
        _tmp_data_10 <= _tmp_data_5 < _tmp_data_6;
      end 
      if(_tmp_valid_10 && _tmp_ready_10) begin
        _tmp_valid_10 <= 0;
      end 
      if((_tmp_ready_10 || !_tmp_valid_10) && (xready && xready)) begin
        _tmp_valid_10 <= xvalid && xvalid;
      end 
      if((_tmp_ready_11 || !_tmp_valid_11) && xready && xvalid) begin
        _tmp_data_11 <= xdata;
      end 
      if(_tmp_valid_11 && _tmp_ready_11) begin
        _tmp_valid_11 <= 0;
      end 
      if((_tmp_ready_11 || !_tmp_valid_11) && xready) begin
        _tmp_valid_11 <= xvalid;
      end 
      if((_tmp_ready_12 || !_tmp_valid_12) && xready && xvalid) begin
        _tmp_data_12 <= _tmp_data_0;
      end 
      if(_tmp_valid_12 && _tmp_ready_12) begin
        _tmp_valid_12 <= 0;
      end 
      if((_tmp_ready_12 || !_tmp_valid_12) && xready) begin
        _tmp_valid_12 <= xvalid;
      end 
      if((_tmp_ready_13 || !_tmp_valid_13) && xready && xvalid) begin
        _tmp_data_13 <= _tmp_data_1;
      end 
      if(_tmp_valid_13 && _tmp_ready_13) begin
        _tmp_valid_13 <= 0;
      end 
      if((_tmp_ready_13 || !_tmp_valid_13) && xready) begin
        _tmp_valid_13 <= xvalid;
      end 
      if((_tmp_ready_14 || !_tmp_valid_14) && xready && xvalid) begin
        _tmp_data_14 <= _tmp_data_2;
      end 
      if(_tmp_valid_14 && _tmp_ready_14) begin
        _tmp_valid_14 <= 0;
      end 
      if((_tmp_ready_14 || !_tmp_valid_14) && xready) begin
        _tmp_valid_14 <= xvalid;
      end 
      if((_tmp_ready_15 || !_tmp_valid_15) && xready && xvalid) begin
        _tmp_data_15 <= _tmp_data_3;
      end 
      if(_tmp_valid_15 && _tmp_ready_15) begin
        _tmp_valid_15 <= 0;
      end 
      if((_tmp_ready_15 || !_tmp_valid_15) && xready) begin
        _tmp_valid_15 <= xvalid;
      end 
      if((_tmp_ready_16 || !_tmp_valid_16) && xready && xvalid) begin
        _tmp_data_16 <= _tmp_data_4;
      end 
      if(_tmp_valid_16 && _tmp_ready_16) begin
        _tmp_valid_16 <= 0;
      end 
      if((_tmp_ready_16 || !_tmp_valid_16) && xready) begin
        _tmp_valid_16 <= xvalid;
      end 
      if((_tmp_ready_17 || !_tmp_valid_17) && xready && xvalid) begin
        _tmp_data_17 <= _tmp_data_5;
      end 
      if(_tmp_valid_17 && _tmp_ready_17) begin
        _tmp_valid_17 <= 0;
      end 
      if((_tmp_ready_17 || !_tmp_valid_17) && xready) begin
        _tmp_valid_17 <= xvalid;
      end 
      if((_tmp_ready_18 || !_tmp_valid_18) && xready && xvalid) begin
        _tmp_data_18 <= _tmp_data_6;
      end 
      if(_tmp_valid_18 && _tmp_ready_18) begin
        _tmp_valid_18 <= 0;
      end 
      if((_tmp_ready_18 || !_tmp_valid_18) && xready) begin
        _tmp_valid_18 <= xvalid;
      end 
      if((_tmp_ready_19 || !_tmp_valid_19) && (_tmp_ready_7 && _tmp_ready_11 && _tmp_ready_12) && (_tmp_valid_7 && _tmp_valid_11 && _tmp_valid_12)) begin
        _tmp_data_19 <= (_tmp_data_7)? _tmp_data_11 : _tmp_data_12;
      end 
      if(_tmp_valid_19 && _tmp_ready_19) begin
        _tmp_valid_19 <= 0;
      end 
      if((_tmp_ready_19 || !_tmp_valid_19) && (_tmp_ready_7 && _tmp_ready_11 && _tmp_ready_12)) begin
        _tmp_valid_19 <= _tmp_valid_7 && _tmp_valid_11 && _tmp_valid_12;
      end 
      if((_tmp_ready_20 || !_tmp_valid_20) && (_tmp_ready_8 && _tmp_ready_13 && _tmp_ready_14) && (_tmp_valid_8 && _tmp_valid_13 && _tmp_valid_14)) begin
        _tmp_data_20 <= (_tmp_data_8)? _tmp_data_13 : _tmp_data_14;
      end 
      if(_tmp_valid_20 && _tmp_ready_20) begin
        _tmp_valid_20 <= 0;
      end 
      if((_tmp_ready_20 || !_tmp_valid_20) && (_tmp_ready_8 && _tmp_ready_13 && _tmp_ready_14)) begin
        _tmp_valid_20 <= _tmp_valid_8 && _tmp_valid_13 && _tmp_valid_14;
      end 
      if((_tmp_ready_21 || !_tmp_valid_21) && (_tmp_ready_9 && _tmp_ready_15 && _tmp_ready_16) && (_tmp_valid_9 && _tmp_valid_15 && _tmp_valid_16)) begin
        _tmp_data_21 <= (_tmp_data_9)? _tmp_data_15 : _tmp_data_16;
      end 
      if(_tmp_valid_21 && _tmp_ready_21) begin
        _tmp_valid_21 <= 0;
      end 
      if((_tmp_ready_21 || !_tmp_valid_21) && (_tmp_ready_9 && _tmp_ready_15 && _tmp_ready_16)) begin
        _tmp_valid_21 <= _tmp_valid_9 && _tmp_valid_15 && _tmp_valid_16;
      end 
      if((_tmp_ready_22 || !_tmp_valid_22) && (_tmp_ready_10 && _tmp_ready_17 && _tmp_ready_18) && (_tmp_valid_10 && _tmp_valid_17 && _tmp_valid_18)) begin
        _tmp_data_22 <= (_tmp_data_10)? _tmp_data_17 : _tmp_data_18;
      end 
      if(_tmp_valid_22 && _tmp_ready_22) begin
        _tmp_valid_22 <= 0;
      end 
      if((_tmp_ready_22 || !_tmp_valid_22) && (_tmp_ready_10 && _tmp_ready_17 && _tmp_ready_18)) begin
        _tmp_valid_22 <= _tmp_valid_10 && _tmp_valid_17 && _tmp_valid_18;
      end 
      if((_tmp_ready_23 || !_tmp_valid_23) && (_tmp_ready_19 && _tmp_ready_20) && (_tmp_valid_19 && _tmp_valid_20)) begin
        _tmp_data_23 <= _tmp_data_19 < _tmp_data_20;
      end 
      if(_tmp_valid_23 && _tmp_ready_23) begin
        _tmp_valid_23 <= 0;
      end 
      if((_tmp_ready_23 || !_tmp_valid_23) && (_tmp_ready_19 && _tmp_ready_20)) begin
        _tmp_valid_23 <= _tmp_valid_19 && _tmp_valid_20;
      end 
      if((_tmp_ready_24 || !_tmp_valid_24) && (_tmp_ready_21 && _tmp_ready_22) && (_tmp_valid_21 && _tmp_valid_22)) begin
        _tmp_data_24 <= _tmp_data_21 < _tmp_data_22;
      end 
      if(_tmp_valid_24 && _tmp_ready_24) begin
        _tmp_valid_24 <= 0;
      end 
      if((_tmp_ready_24 || !_tmp_valid_24) && (_tmp_ready_21 && _tmp_ready_22)) begin
        _tmp_valid_24 <= _tmp_valid_21 && _tmp_valid_22;
      end 
      if((_tmp_ready_25 || !_tmp_valid_25) && _tmp_ready_19 && _tmp_valid_19) begin
        _tmp_data_25 <= _tmp_data_19;
      end 
      if(_tmp_valid_25 && _tmp_ready_25) begin
        _tmp_valid_25 <= 0;
      end 
      if((_tmp_ready_25 || !_tmp_valid_25) && _tmp_ready_19) begin
        _tmp_valid_25 <= _tmp_valid_19;
      end 
      if((_tmp_ready_26 || !_tmp_valid_26) && _tmp_ready_20 && _tmp_valid_20) begin
        _tmp_data_26 <= _tmp_data_20;
      end 
      if(_tmp_valid_26 && _tmp_ready_26) begin
        _tmp_valid_26 <= 0;
      end 
      if((_tmp_ready_26 || !_tmp_valid_26) && _tmp_ready_20) begin
        _tmp_valid_26 <= _tmp_valid_20;
      end 
      if((_tmp_ready_27 || !_tmp_valid_27) && _tmp_ready_21 && _tmp_valid_21) begin
        _tmp_data_27 <= _tmp_data_21;
      end 
      if(_tmp_valid_27 && _tmp_ready_27) begin
        _tmp_valid_27 <= 0;
      end 
      if((_tmp_ready_27 || !_tmp_valid_27) && _tmp_ready_21) begin
        _tmp_valid_27 <= _tmp_valid_21;
      end 
      if((_tmp_ready_28 || !_tmp_valid_28) && _tmp_ready_22 && _tmp_valid_22) begin
        _tmp_data_28 <= _tmp_data_22;
      end 
      if(_tmp_valid_28 && _tmp_ready_28) begin
        _tmp_valid_28 <= 0;
      end 
      if((_tmp_ready_28 || !_tmp_valid_28) && _tmp_ready_22) begin
        _tmp_valid_28 <= _tmp_valid_22;
      end 
      if((_tmp_ready_29 || !_tmp_valid_29) && (_tmp_ready_23 && _tmp_ready_25 && _tmp_ready_26) && (_tmp_valid_23 && _tmp_valid_25 && _tmp_valid_26)) begin
        _tmp_data_29 <= (_tmp_data_23)? _tmp_data_25 : _tmp_data_26;
      end 
      if(_tmp_valid_29 && _tmp_ready_29) begin
        _tmp_valid_29 <= 0;
      end 
      if((_tmp_ready_29 || !_tmp_valid_29) && (_tmp_ready_23 && _tmp_ready_25 && _tmp_ready_26)) begin
        _tmp_valid_29 <= _tmp_valid_23 && _tmp_valid_25 && _tmp_valid_26;
      end 
      if((_tmp_ready_30 || !_tmp_valid_30) && (_tmp_ready_24 && _tmp_ready_27 && _tmp_ready_28) && (_tmp_valid_24 && _tmp_valid_27 && _tmp_valid_28)) begin
        _tmp_data_30 <= (_tmp_data_24)? _tmp_data_27 : _tmp_data_28;
      end 
      if(_tmp_valid_30 && _tmp_ready_30) begin
        _tmp_valid_30 <= 0;
      end 
      if((_tmp_ready_30 || !_tmp_valid_30) && (_tmp_ready_24 && _tmp_ready_27 && _tmp_ready_28)) begin
        _tmp_valid_30 <= _tmp_valid_24 && _tmp_valid_27 && _tmp_valid_28;
      end 
      if((_tmp_ready_31 || !_tmp_valid_31) && (_tmp_ready_29 && _tmp_ready_30) && (_tmp_valid_29 && _tmp_valid_30)) begin
        _tmp_data_31 <= _tmp_data_29 < _tmp_data_30;
      end 
      if(_tmp_valid_31 && _tmp_ready_31) begin
        _tmp_valid_31 <= 0;
      end 
      if((_tmp_ready_31 || !_tmp_valid_31) && (_tmp_ready_29 && _tmp_ready_30)) begin
        _tmp_valid_31 <= _tmp_valid_29 && _tmp_valid_30;
      end 
      if((_tmp_ready_32 || !_tmp_valid_32) && _tmp_ready_29 && _tmp_valid_29) begin
        _tmp_data_32 <= _tmp_data_29;
      end 
      if(_tmp_valid_32 && _tmp_ready_32) begin
        _tmp_valid_32 <= 0;
      end 
      if((_tmp_ready_32 || !_tmp_valid_32) && _tmp_ready_29) begin
        _tmp_valid_32 <= _tmp_valid_29;
      end 
      if((_tmp_ready_33 || !_tmp_valid_33) && _tmp_ready_30 && _tmp_valid_30) begin
        _tmp_data_33 <= _tmp_data_30;
      end 
      if(_tmp_valid_33 && _tmp_ready_33) begin
        _tmp_valid_33 <= 0;
      end 
      if((_tmp_ready_33 || !_tmp_valid_33) && _tmp_ready_30) begin
        _tmp_valid_33 <= _tmp_valid_30;
      end 
      if((_tmp_ready_34 || !_tmp_valid_34) && (_tmp_ready_31 && _tmp_ready_32 && _tmp_ready_33) && (_tmp_valid_31 && _tmp_valid_32 && _tmp_valid_33)) begin
        _tmp_data_34 <= (_tmp_data_31)? _tmp_data_32 : _tmp_data_33;
      end 
      if(_tmp_valid_34 && _tmp_ready_34) begin
        _tmp_valid_34 <= 0;
      end 
      if((_tmp_ready_34 || !_tmp_valid_34) && (_tmp_ready_31 && _tmp_ready_32 && _tmp_ready_33)) begin
        _tmp_valid_34 <= _tmp_valid_31 && _tmp_valid_32 && _tmp_valid_33;
      end 
    end
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = dataflow_movmin.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function

import functools

from veriloggen.dataflow import *
from veriloggen.seq.seq import make_condition
from veriloggen.fsm.fsm import FSM
from veriloggen.seq.seq import Seq
import veriloggen.dataflow.dtypes as dtypes

from . import compiler


class Stream(vtypes.VeriloggenNode):
    __intrinsics__ = ('run', 'join', 'done')

    def __init__(self, m, name, clk, rst, func):
        self.m = m
        self.name = name
        self.clk = clk
        self.rst = rst
        self.func = func

        self.df = DataflowManager(self.m, self.clk, self.rst)
        self.seq = Seq(self.m, 'name', self.clk, self.rst)
        self.start_cond = None
        self.done_flags = []

    def run(self, fsm, *args, **kwargs):
        self.start_cond = fsm.state == fsm.current
        self.func(self, *args)

        fsm.goto_next()

        return 0

    def join(self, fsm):
        done = None

        for flag in self.done_flags:
            done = make_condition(done, flag)

        fsm.If(done).goto_next()
        return 0

    def done(self, fsm):
        done = None

        for flag in self.done_flags:
            done = make_condition(done, flag)

        return done

    def read_sequential(self, obj, start_addr, size, point=0, signed=False,
                        port=0, with_last=False):
        flag = self.m.Reg(compiler._tmp_name('_'.join(['', self.name, 'flag'])),
                          initval=0)
        self.done_flags.append(flag)

        fsm = FSM(self.m, compiler._tmp_name('_'.join(['', self.name, 'fsm'])),
                  self.clk, self.rst)
        fsm.If(self.start_cond)(
            flag(0)
        )
        fsm.If(self.start_cond).goto_next()


        rdata, rlast, done = obj.read_dataflow(
            port, start_addr, size, cond=fsm, point=point, signed=signed)
        fsm.goto_next()

        fsm.If(done)(
            flag(1)
        )

        fsm.If(done).goto_init()

        if with_last:
            return rdata, rlast

        return rdata

    def write_sequential(self, obj, start_addr, size, value, when=None, port=0):
        flag = self.m.Reg(compiler._tmp_name('_'.join(['', self.name, 'flag'])),
                          initval=0)
        self.done_flags.append(flag)

        fsm = FSM(self.m, compiler._tmp_name('_'.join(['', self.name, 'fsm'])),
                  self.clk, self.rst)
        fsm.If(self.start_cond)(
            flag(0)
        )
        fsm.If(self.start_cond).goto_next()

        done = obj.write_dataflow(
            port, start_addr, value, size, cond=fsm, when=when)
        fsm.goto_next()

        fsm.If(done)(
            flag(1)
        )

        fsm.If(done).goto_init()

        return 0

    def __getattr__(self, attr):
        try:
            return object.__getattr__(self, attr)

        except AttributeError as e:
            if attr.startswith('__') or attr not in dir(dtypes):
                raise e

            func = getattr(dtypes, attr)

            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                v = func(*args, **kwargs)
                if isinstance(v, (tuple, list)):
                    for item in v:
                        self._set_info(item)
                else:
                    self._set_info(v)
                return v

            return wrapper

    def _set_info(self, v):
        if isinstance(v, dtypes._Numeric):
            v._set_module(self.m)
            v._set_df(self.df)
            v._set_seq(self.seq)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from veriloggen import *

def mkLed():
    m = Module('blinkled')
    interval = m.Parameter('INTERVAL', 16)
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.OutputReg('LED', 8, initval=0)
    count = m.Reg('count', 32, initval=0)
    
    seq = Seq(m, 'seq', clk, rst)
    seq.add( Systask('display', 'LED:%d count:%d', led, count) )
    seq.add( count(count + 1), cond=count<interval-1 )
    seq.add( count(0), cond=count==interval-1 )
    seq.add( led(led + 1), cond=count==interval-1 )

    seq.make_always()
    
    return m

def mkTest():
    m = Module('test')

    # target instance
    led = mkLed()
    
    # copy paras and ports
    params = m.copy_params(led)
    ports = m.copy_sim_ports(led)

    clk = ports['CLK']
    rst = ports['RST']
    
    uut = m.Instance(led, 'uut',
                     params=m.connect_params(led),
                     ports=m.connect_ports(led))
    
    #simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(1000),
        Systask('finish'),
    )

    return m
    
if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog()
    print(verilog)



from __future__ import absolute_import
from __future__ import print_function

import veriloggen.core.vtypes as vtypes
from veriloggen.core.module import Module
from veriloggen.seq.seq import Seq
from veriloggen.dataflow.dataflow import DataflowManager
from veriloggen.dataflow.dtypes import make_condition
from . import util


class RAMInterface(object):
    _I = 'Reg'
    _O = 'Wire'

    def __init__(self, m, name=None, datawidth=32, addrwidth=10,
                 itype=None, otype=None,
                 p_addr='addr', p_rdata='rdata',
                 p_wdata='wdata', p_wenable='wenable',
                 p_enable='enable',
                 with_enable=False, index=None):

        if itype is None:
            itype = self._I
        if otype is None:
            otype = self._O

        self.m = m

        name_addr = p_addr if name is None else '_'.join([name, p_addr])
        name_rdata = p_rdata if name is None else '_'.join([name, p_rdata])
        name_wdata = p_wdata if name is None else '_'.join([name, p_wdata])
        name_wenable = (
            p_wenable if name is None else '_'.join([name, p_wenable]))
        if with_enable:
            name_enable = (
                p_enable if name is None else '_'.join([name, p_enable]))

        if index is not None:
            name_addr = name_addr + str(index)
            name_rdata = name_rdata + str(index)
            name_wdata = name_wdata + str(index)
            name_wenable = name_wenable + str(index)
            if with_enable:
                name_enable = name_enable + str(index)

        self.addr = util.make_port(m, itype, name_addr, addrwidth, initval=0)
        self.rdata = util.make_port(m, otype, name_rdata, datawidth, initval=0)
        self.wdata = util.make_port(m, itype, name_wdata, datawidth, initval=0)
        self.wenable = util.make_port(m, itype, name_wenable, initval=0)
        if with_enable:
            self.enable = util.make_port(m, itype, name_enable, initval=0)

    def connect(self, targ):
        util.connect_port(self.addr, targ.addr)
        util.connect_port(targ.rdata, self.rdata)
        util.connect_port(self.wdata, targ.wdata)
        util.connect_port(self.wenable, targ.wenable)
        if hasattr(self, 'enable'):
            if hasattr(targ, 'enable'):
                util.connect_port(self.enable, targ.enable)
            else:
                util.connect_port(self.enable, 1)
        else:
            if hasattr(targ, 'enable'):
                raise ValueError('no enable port')


class RAMSlaveInterface(RAMInterface):
    _I = 'Input'
    _O = 'Output'


class RAMMasterInterface(RAMInterface):
    _I = 'Output'
    _O = 'Input'


def mkRAMDefinition(name, datawidth=32, addrwidth=10, numports=2, sync=True, with_enable=False):
    m = Module(name)
    clk = m.Input('CLK')

    interfaces = []

    for i in range(numports):
        interface = RAMSlaveInterface(
            m, name + '_%d' % i, datawidth, addrwidth, with_enable=with_enable)
        interface.delay_addr = m.Reg(name + '_%d_daddr' % i, addrwidth)

        interfaces.append(interface)

    mem = m.Reg('mem', datawidth, length=2**addrwidth)

    for interface in interfaces:
        body = [
            vtypes.If(interface.wenable)(
                mem[interface.addr](interface.wdata)
            ),
            interface.delay_addr(interface.addr)]

        if with_enable:
            body = vtypes.If(interface.enable)(*body)

        m.Always(vtypes.Posedge(clk))(
            body
        )

        if sync:
            m.Assign(interface.rdata(mem[interface.delay_addr]))
        else:
            m.Assign(interface.rdata(mem[interface.addr]))

    return m


class _RAM(object):

    def __init__(self, m, name, clk,
                 datawidth=32, addrwidth=10, numports=1, sync=True, with_enable=False):

        self.m = m
        self.name = name
        self.clk = clk
        self.with_enable = with_enable

        self.interfaces = [RAMInterface(m, name + '_%d' % i, datawidth, addrwidth,
                                        itype='Wire', otype='Wire', with_enable=with_enable)
                           for i in range(numports)]

        ram_def = mkRAMDefinition(
            name, datawidth, addrwidth, numports, sync, with_enable)

        self.m.Instance(ram_def, name,
                        params=(), ports=m.connect_ports(ram_def))

    def connect(self, port, addr, wdata, wenable, enable=None):
        self.m.Assign(self.interfaces[port].addr(addr))
        self.m.Assign(self.interfaces[port].wdata(wdata))
        self.m.Assign(self.interfaces[port].wenable(wenable))
        if self.with_enable:
            self.m.Assign(self.interfaces[port].enable(enable))

    def rdata(self, port):
        return self.interfaces[port].rdata


class SyncRAM(_RAM):

    def __init__(self, m, name, clk,
                 datawidth=32, addrwidth=10, numports=1, with_enable=False):
        _RAM.__init__(self, m, name, clk,
                      datawidth, addrwidth, numports, sync=True, with_enable=with_enable)


class AsyncRAM(_RAM):

    def __init__(self, m, name, clk,
                 datawidth=32, addrwidth=10, numports=1, with_enable=False):
        _RAM.__init__(self, m, name, clk,
                      datawidth, addrwidth, numports, sync=False)


#-------------------------------------------------------------------------
class SyncRAMManager (object):

    def __init__(self, m, name, clk, rst,
                 datawidth=32, addrwidth=10, numports=1,
                 nodataflow=False):

        self.m = m
        self.name = name
        self.clk = clk
        self.rst = rst
        self.datawidth = datawidth
        self.addrwidth = addrwidth

        self.interfaces = [RAMInterface(m, name + '_%d' % i, datawidth, addrwidth)
                           for i in range(numports)]

        self.definition = mkRAMDefinition(name, datawidth, addrwidth, numports)

        self.inst = self.m.Instance(self.definition, 'inst_' + name,
                                    ports=m.connect_ports(self.definition))

        self.seq = Seq(m, name, clk, rst)

        if nodataflow:
            self.df = None
        else:
            self.df = DataflowManager(self.m, self.clk, self.rst)

        self._write_disabled = [False for i in range(numports)]

    def __getitem__(self, index):
        return self.interfaces[index]

    def disable_write(self, port):
        self.seq(
            self.interfaces[port].wdata(0),
            self.interfaces[port].wenable(0)
        )
        self._write_disabled[port] = True

    def write(self, port, addr, wdata, cond=None):
        """ 
        @return None
        """

        if self._write_disabled[port]:
            raise TypeError('Write disabled.')

        if cond is not None:
            self.seq.If(cond)

        self.seq(
            self.interfaces[port].addr(addr),
            self.interfaces[port].wdata(wdata),
            self.interfaces[port].wenable(1)
        )

        self.seq.Then().Delay(1)(
            self.interfaces[port].wenable(0)
        )

    def write_dataflow(self, port, addr, data, length=1, cond=None, when=None):
        """ 
        @return done
        """

        if self._write_disabled[port]:
            raise TypeError('Write disabled.')

        counter = self.m.TmpReg(length.bit_length() + 1, initval=0)
        last = self.m.TmpReg(initval=0)

        ext_cond = make_condition(cond)
        data_cond = make_condition(counter > 0, vtypes.Not(last))
        all_cond = make_condition(data_cond, ext_cond)
        raw_data, raw_valid = data.read(cond=data_cond)

        when_cond = make_condition(when, ready=data_cond)
        if when_cond is not None:
            raw_valid = vtypes.Ands(when_cond, raw_valid)

        self.seq.If(make_condition(ext_cond, counter == 0))(
            self.interfaces[port].addr(addr - 1),
            counter(length),
        )

        self.seq.If(make_condition(raw_valid, counter > 0))(
            self.interfaces[port].addr.inc(),
            self.interfaces[port].wdata(raw_data),
            self.interfaces[port].wenable(1),
            counter.dec()
        )

        self.seq.If(make_condition(raw_valid, counter == 1))(
            last(1)
        )

        # de-assert
        self.seq.Delay(1)(
            self.interfaces[port].wenable(0),
            last(0)
        )

        done = last

        return done

    def read(self, port, addr, cond=None):
        """ 
        @return data, valid
        """

        if cond is not None:
            self.seq.If(cond)

        self.seq(
            self.interfaces[port].addr(addr)
        )

        rdata = self.interfaces[port].rdata
        rvalid = self.m.TmpReg(initval=0)

        self.seq.Then().Delay(1)(
            rvalid(1)
        )

        self.seq.Then().Delay(2)(
            rvalid(0)
        )

        return rdata, rvalid

    def read_dataflow(self, port, addr, length=1, cond=None, point=0, signed=False):
        """ 
        @return data, last, done
        """

        data_valid = self.m.TmpReg(initval=0)
        last_valid = self.m.TmpReg(initval=0)
        data_ready = self.m.TmpWire()
        last_ready = self.m.TmpWire()
        data_ready.assign(1)
        last_ready.assign(1)

        data_ack = vtypes.Ors(data_ready, vtypes.Not(data_valid))
        last_ack = vtypes.Ors(last_ready, vtypes.Not(last_valid))

        ext_cond = make_condition(cond)
        data_cond = make_condition(data_ack, last_ack)
        prev_data_cond = self.seq.Prev(data_cond, 1)
        all_cond = make_condition(data_cond, ext_cond)

        data = self.m.TmpWireLike(self.interfaces[port].rdata)

        prev_data = self.seq.Prev(data, 1)
        data.assign(vtypes.Mux(prev_data_cond,
                               self.interfaces[port].rdata, prev_data))

        counter = self.m.TmpReg(length.bit_length() + 1, initval=0)

        next_valid_on = self.m.TmpReg(initval=0)
        next_valid_off = self.m.TmpReg(initval=0)

        next_last = self.m.TmpReg(initval=0)
        last = self.m.TmpReg(initval=0)

        self.seq.If(make_condition(data_cond, next_valid_off))(
            last(0),
            data_valid(0),
            last_valid(0),
            next_valid_off(0)
        )

        self.seq.If(make_condition(data_cond, next_valid_on))(
            data_valid(1),
            last_valid(1),
            last(next_last),
            next_last(0),
            next_valid_on(0),
            next_valid_off(1)
        )

        self.seq.If(make_condition(ext_cond, counter == 0,
                                   vtypes.Not(next_last), vtypes.Not(last)))(
            self.interfaces[port].addr(addr),
            counter(length - 1),
            next_valid_on(1),
        )

        self.seq.If(make_condition(data_cond, counter > 0))(
            self.interfaces[port].addr.inc(),
            counter.dec(),
            next_valid_on(1),
            next_last(0)
        )

        self.seq.If(make_condition(data_cond, counter == 1))(
            next_last(1)
        )

        df = self.df if self.df is not None else dataflow

        df_data = df.Variable(data, data_valid, data_ready,
                              point=point, signed=signed)
        df_last = df.Variable(last, last_valid, last_ready, width=1)
        done = last

        return df_data, df_last, done



from __future__ import absolute_import
from __future__ import print_function

import veriloggen.core.vtypes as vtypes
import veriloggen.seq.seq as seq

t_Input = 'Input'
t_Output = 'Output'
t_Wire = 'Wire'
t_Reg = 'Reg'
t_OutputReg = 'OutputReg'


def swap_type(cls):
    return cls._O, cls._I


def make_port(m, _type, *args, **kwargs):
    if 'initval' in kwargs and 'Reg' not in _type:
        del kwargs['initval']
    return getattr(m, _type)(*args, **kwargs)


def connect_port(left, right):
    if isinstance(left, vtypes.Reg):
        wire_left = left.module.TmpWireLike(left)
        wire_left.assign(right)
        left.module.Always()(left(wire_left, blk=True))
    else:
        left.assign(right)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import dataflow_lut

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  reg [32-1:0] xdata;
  reg xvalid;
  wire xready;
  reg [32-1:0] ydata;
  reg yvalid;
  wire yready;
  wire [32-1:0] zdata;
  wire zvalid;
  reg zready;

  main
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .xdata(xdata),
    .xvalid(xvalid),
    .xready(xready),
    .ydata(ydata),
    .yvalid(yvalid),
    .yready(yready),
    .zdata(zdata),
    .zvalid(zvalid),
    .zready(zready)
  );

  reg reset_done;

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    reset_done = 0;
    xdata = 0;
    xvalid = 0;
    ydata = 0;
    yvalid = 0;
    zready = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    reset_done = 1;
    @(posedge CLK);
    #1;
    #10000;
    $finish;
  end

  reg [32-1:0] xfsm;
  localparam xfsm_init = 0;
  reg [32-1:0] _tmp_0;
  localparam xfsm_1 = 1;
  localparam xfsm_2 = 2;
  localparam xfsm_3 = 3;
  localparam xfsm_4 = 4;
  localparam xfsm_5 = 5;
  localparam xfsm_6 = 6;
  localparam xfsm_7 = 7;
  localparam xfsm_8 = 8;
  localparam xfsm_9 = 9;
  localparam xfsm_10 = 10;
  localparam xfsm_11 = 11;
  localparam xfsm_12 = 12;
  localparam xfsm_13 = 13;
  localparam xfsm_14 = 14;
  localparam xfsm_15 = 15;
  localparam xfsm_16 = 16;
  localparam xfsm_17 = 17;
  localparam xfsm_18 = 18;
  localparam xfsm_19 = 19;
  localparam xfsm_20 = 20;
  localparam xfsm_21 = 21;
  localparam xfsm_22 = 22;
  localparam xfsm_23 = 23;
  localparam xfsm_24 = 24;

  always @(posedge CLK) begin
    if(RST) begin
      xfsm <= xfsm_init;
      _tmp_0 <= 0;
    end else begin
      case(xfsm)
        xfsm_init: begin
          xvalid <= 0;
          if(reset_done) begin
            xfsm <= xfsm_1;
          end 
        end
        xfsm_1: begin
          xfsm <= xfsm_2;
        end
        xfsm_2: begin
          xfsm <= xfsm_3;
        end
        xfsm_3: begin
          xfsm <= xfsm_4;
        end
        xfsm_4: begin
          xfsm <= xfsm_5;
        end
        xfsm_5: begin
          xfsm <= xfsm_6;
        end
        xfsm_6: begin
          xfsm <= xfsm_7;
        end
        xfsm_7: begin
          xfsm <= xfsm_8;
        end
        xfsm_8: begin
          xfsm <= xfsm_9;
        end
        xfsm_9: begin
          xfsm <= xfsm_10;
        end
        xfsm_10: begin
          xfsm <= xfsm_11;
        end
        xfsm_11: begin
          xvalid <= 1;
          xfsm <= xfsm_12;
        end
        xfsm_12: begin
          if(xready) begin
            xdata <= xdata + 1;
          end 
          if(xready) begin
            _tmp_0 <= _tmp_0 + 1;
          end 
          if((_tmp_0 == 5) && xready) begin
            xvalid <= 0;
          end 
          if((_tmp_0 == 5) && xready) begin
            xfsm <= xfsm_13;
          end 
        end
        xfsm_13: begin
          xfsm <= xfsm_14;
        end
        xfsm_14: begin
          xfsm <= xfsm_15;
        end
        xfsm_15: begin
          xfsm <= xfsm_16;
        end
        xfsm_16: begin
          xfsm <= xfsm_17;
        end
        xfsm_17: begin
          xfsm <= xfsm_18;
        end
        xfsm_18: begin
          xfsm <= xfsm_19;
        end
        xfsm_19: begin
          xfsm <= xfsm_20;
        end
        xfsm_20: begin
          xfsm <= xfsm_21;
        end
        xfsm_21: begin
          xfsm <= xfsm_22;
        end
        xfsm_22: begin
          xfsm <= xfsm_23;
        end
        xfsm_23: begin
          xvalid <= 1;
          if(xready) begin
            xdata <= xdata + 1;
          end 
          if(xready) begin
            _tmp_0 <= _tmp_0 + 1;
          end 
          if((_tmp_0 == 10) && xready) begin
            xvalid <= 0;
          end 
          if((_tmp_0 == 10) && xready) begin
            xfsm <= xfsm_24;
          end 
        end
      endcase
    end
  end

  reg [32-1:0] yfsm;
  localparam yfsm_init = 0;
  reg [32-1:0] _tmp_1;
  localparam yfsm_1 = 1;
  localparam yfsm_2 = 2;
  localparam yfsm_3 = 3;
  localparam yfsm_4 = 4;
  localparam yfsm_5 = 5;
  localparam yfsm_6 = 6;
  localparam yfsm_7 = 7;
  localparam yfsm_8 = 8;
  localparam yfsm_9 = 9;
  localparam yfsm_10 = 10;
  localparam yfsm_11 = 11;
  localparam yfsm_12 = 12;
  localparam yfsm_13 = 13;
  localparam yfsm_14 = 14;
  localparam yfsm_15 = 15;
  localparam yfsm_16 = 16;
  localparam yfsm_17 = 17;
  localparam yfsm_18 = 18;
  localparam yfsm_19 = 19;
  localparam yfsm_20 = 20;
  localparam yfsm_21 = 21;
  localparam yfsm_22 = 22;
  localparam yfsm_23 = 23;
  localparam yfsm_24 = 24;
  localparam yfsm_25 = 25;
  localparam yfsm_26 = 26;
  localparam yfsm_27 = 27;
  localparam yfsm_28 = 28;
  localparam yfsm_29 = 29;
  localparam yfsm_30 = 30;
  localparam yfsm_31 = 31;
  localparam yfsm_32 = 32;
  localparam yfsm_33 = 33;
  localparam yfsm_34 = 34;
  localparam yfsm_35 = 35;
  localparam yfsm_36 = 36;
  localparam yfsm_37 = 37;
  localparam yfsm_38 = 38;
  localparam yfsm_39 = 39;
  localparam yfsm_40 = 40;
  localparam yfsm_41 = 41;
  localparam yfsm_42 = 42;
  localparam yfsm_43 = 43;
  localparam yfsm_44 = 44;

  always @(posedge CLK) begin
    if(RST) begin
      yfsm <= yfsm_init;
      _tmp_1 <= 0;
    end else begin
      case(yfsm)
        yfsm_init: begin
          yvalid <= 0;
          if(reset_done) begin
            yfsm <= yfsm_1;
          end 
        end
        yfsm_1: begin
          yfsm <= yfsm_2;
        end
        yfsm_2: begin
          yfsm <= yfsm_3;
        end
        yfsm_3: begin
          yfsm <= yfsm_4;
        end
        yfsm_4: begin
          yfsm <= yfsm_5;
        end
        yfsm_5: begin
          yfsm <= yfsm_6;
        end
        yfsm_6: begin
          yfsm <= yfsm_7;
        end
        yfsm_7: begin
          yfsm <= yfsm_8;
        end
        yfsm_8: begin
          yfsm <= yfsm_9;
        end
        yfsm_9: begin
          yfsm <= yfsm_10;
        end
        yfsm_10: begin
          yfsm <= yfsm_11;
        end
        yfsm_11: begin
          yfsm <= yfsm_12;
        end
        yfsm_12: begin
          yfsm <= yfsm_13;
        end
        yfsm_13: begin
          yfsm <= yfsm_14;
        end
        yfsm_14: begin
          yfsm <= yfsm_15;
        end
        yfsm_15: begin
          yfsm <= yfsm_16;
        end
        yfsm_16: begin
          yfsm <= yfsm_17;
        end
        yfsm_17: begin
          yfsm <= yfsm_18;
        end
        yfsm_18: begin
          yfsm <= yfsm_19;
        end
        yfsm_19: begin
          yfsm <= yfsm_20;
        end
        yfsm_20: begin
          yfsm <= yfsm_21;
        end
        yfsm_21: begin
          yvalid <= 1;
          yfsm <= yfsm_22;
        end
        yfsm_22: begin
          if(yready) begin
            ydata <= ydata + 2;
          end 
          if(yready) begin
            _tmp_1 <= _tmp_1 + 1;
          end 
          if((_tmp_1 == 5) && yready) begin
            yvalid <= 0;
          end 
          if((_tmp_1 == 5) && yready) begin
            yfsm <= yfsm_23;
          end 
        end
        yfsm_23: begin
          yfsm <= yfsm_24;
        end
        yfsm_24: begin
          yfsm <= yfsm_25;
        end
        yfsm_25: begin
          yfsm <= yfsm_26;
        end
        yfsm_26: begin
          yfsm <= yfsm_27;
        end
        yfsm_27: begin
          yfsm <= yfsm_28;
        end
        yfsm_28: begin
          yfsm <= yfsm_29;
        end
        yfsm_29: begin
          yfsm <= yfsm_30;
        end
        yfsm_30: begin
          yfsm <= yfsm_31;
        end
        yfsm_31: begin
          yfsm <= yfsm_32;
        end
        yfsm_32: begin
          yfsm <= yfsm_33;
        end
        yfsm_33: begin
          yfsm <= yfsm_34;
        end
        yfsm_34: begin
          yfsm <= yfsm_35;
        end
        yfsm_35: begin
          yfsm <= yfsm_36;
        end
        yfsm_36: begin
          yfsm <= yfsm_37;
        end
        yfsm_37: begin
          yfsm <= yfsm_38;
        end
        yfsm_38: begin
          yfsm <= yfsm_39;
        end
        yfsm_39: begin
          yfsm <= yfsm_40;
        end
        yfsm_40: begin
          yfsm <= yfsm_41;
        end
        yfsm_41: begin
          yfsm <= yfsm_42;
        end
        yfsm_42: begin
          yfsm <= yfsm_43;
        end
        yfsm_43: begin
          yvalid <= 1;
          if(yready) begin
            ydata <= ydata + 2;
          end 
          if(yready) begin
            _tmp_1 <= _tmp_1 + 1;
          end 
          if((_tmp_1 == 10) && yready) begin
            yvalid <= 0;
          end 
          if((_tmp_1 == 10) && yready) begin
            yfsm <= yfsm_44;
          end 
        end
      endcase
    end
  end

  reg [32-1:0] zfsm;
  localparam zfsm_init = 0;
  localparam zfsm_1 = 1;
  localparam zfsm_2 = 2;
  localparam zfsm_3 = 3;
  localparam zfsm_4 = 4;
  localparam zfsm_5 = 5;
  localparam zfsm_6 = 6;
  localparam zfsm_7 = 7;
  localparam zfsm_8 = 8;

  always @(posedge CLK) begin
    if(RST) begin
      zfsm <= zfsm_init;
    end else begin
      case(zfsm)
        zfsm_init: begin
          zready <= 0;
          if(reset_done) begin
            zfsm <= zfsm_1;
          end 
        end
        zfsm_1: begin
          zfsm <= zfsm_2;
        end
        zfsm_2: begin
          if(zvalid) begin
            zready <= 1;
          end 
          if(zvalid) begin
            zfsm <= zfsm_3;
          end 
        end
        zfsm_3: begin
          zready <= 0;
          zfsm <= zfsm_4;
        end
        zfsm_4: begin
          zready <= 0;
          zfsm <= zfsm_5;
        end
        zfsm_5: begin
          zready <= 0;
          zfsm <= zfsm_6;
        end
        zfsm_6: begin
          zready <= 0;
          zfsm <= zfsm_7;
        end
        zfsm_7: begin
          zready <= 0;
          zfsm <= zfsm_8;
        end
        zfsm_8: begin
          zfsm <= zfsm_2;
        end
      endcase
    end
  end


  always @(posedge CLK) begin
    if(reset_done) begin
      if(xvalid && xready) begin
        $display("xdata=%d", xdata);
      end 
      if(yvalid && yready) begin
        $display("ydata=%d", ydata);
      end 
      if(zvalid && zready) begin
        $display("zdata=%d", zdata);
      end 
    end 
  end


endmodule



module main
(
  input CLK,
  input RST,
  input [32-1:0] xdata,
  input xvalid,
  output xready,
  input [32-1:0] ydata,
  input yvalid,
  output yready,
  output [32-1:0] zdata,
  output zvalid,
  input zready
);

  wire [32-1:0] _tmp_data_0;
  reg _tmp_valid_0;
  wire _tmp_ready_0;
  wire [8-1:0] _tmp_address_0;
  assign _tmp_address_0 = xdata;

  _LUT_0
  LUT_0
  (
    .CLK(CLK),
    .addr(_tmp_address_0),
    .enable((_tmp_ready_0 || !_tmp_valid_0) && xready && xvalid),
    .val(_tmp_data_0)
  );

  assign xready = (_tmp_ready_0 || !_tmp_valid_0) && xvalid;
  assign _tmp_ready_0 = (_tmp_ready_2 || !_tmp_valid_2) && (_tmp_valid_0 && _tmp_valid_1);
  reg [32-1:0] _tmp_data_1;
  reg _tmp_valid_1;
  wire _tmp_ready_1;
  assign yready = (_tmp_ready_1 || !_tmp_valid_1) && yvalid;
  assign _tmp_ready_1 = (_tmp_ready_2 || !_tmp_valid_2) && (_tmp_valid_0 && _tmp_valid_1);
  reg [32-1:0] _tmp_data_2;
  reg _tmp_valid_2;
  wire _tmp_ready_2;
  assign zdata = _tmp_data_2;
  assign zvalid = _tmp_valid_2;
  assign _tmp_ready_2 = zready;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_valid_0 <= 0;
      _tmp_data_1 <= 0;
      _tmp_valid_1 <= 0;
      _tmp_data_2 <= 0;
      _tmp_valid_2 <= 0;
    end else begin
      if(_tmp_valid_0 && _tmp_ready_0) begin
        _tmp_valid_0 <= 0;
      end 
      if((_tmp_ready_0 || !_tmp_valid_0) && xready) begin
        _tmp_valid_0 <= xvalid;
      end 
      if((_tmp_ready_1 || !_tmp_valid_1) && yready && yvalid) begin
        _tmp_data_1 <= ydata;
      end 
      if(_tmp_valid_1 && _tmp_ready_1) begin
        _tmp_valid_1 <= 0;
      end 
      if((_tmp_ready_1 || !_tmp_valid_1) && yready) begin
        _tmp_valid_1 <= yvalid;
      end 
      if((_tmp_ready_2 || !_tmp_valid_2) && (_tmp_ready_0 && _tmp_ready_1) && (_tmp_valid_0 && _tmp_valid_1)) begin
        _tmp_data_2 <= _tmp_data_0 + _tmp_data_1;
      end 
      if(_tmp_valid_2 && _tmp_ready_2) begin
        _tmp_valid_2 <= 0;
      end 
      if((_tmp_ready_2 || !_tmp_valid_2) && (_tmp_ready_0 && _tmp_ready_1)) begin
        _tmp_valid_2 <= _tmp_valid_0 && _tmp_valid_1;
      end 
    end
  end


endmodule



module _LUT_0
(
  input CLK,
  input [8-1:0] addr,
  input enable,
  output reg [32-1:0] val
);


  always @(posedge CLK) begin
    if(enable) begin
      case(addr)
        0: begin
          val <= 0;
        end
        1: begin
          val <= 1;
        end
        2: begin
          val <= 4;
        end
        3: begin
          val <= 9;
        end
        4: begin
          val <= 16;
        end
        5: begin
          val <= 25;
        end
        6: begin
          val <= 36;
        end
        7: begin
          val <= 49;
        end
        8: begin
          val <= 64;
        end
        9: begin
          val <= 81;
        end
        10: begin
          val <= 100;
        end
        11: begin
          val <= 121;
        end
        12: begin
          val <= 144;
        end
        13: begin
          val <= 169;
        end
        14: begin
          val <= 196;
        end
        15: begin
          val <= 225;
        end
        16: begin
          val <= 256;
        end
        17: begin
          val <= 289;
        end
        18: begin
          val <= 324;
        end
        19: begin
          val <= 361;
        end
        20: begin
          val <= 400;
        end
        21: begin
          val <= 441;
        end
        22: begin
          val <= 484;
        end
        23: begin
          val <= 529;
        end
        24: begin
          val <= 576;
        end
        25: begin
          val <= 625;
        end
        26: begin
          val <= 676;
        end
        27: begin
          val <= 729;
        end
        28: begin
          val <= 784;
        end
        29: begin
          val <= 841;
        end
        30: begin
          val <= 900;
        end
        31: begin
          val <= 961;
        end
        32: begin
          val <= 1024;
        end
        33: begin
          val <= 1089;
        end
        34: begin
          val <= 1156;
        end
        35: begin
          val <= 1225;
        end
        36: begin
          val <= 1296;
        end
        37: begin
          val <= 1369;
        end
        38: begin
          val <= 1444;
        end
        39: begin
          val <= 1521;
        end
        40: begin
          val <= 1600;
        end
        41: begin
          val <= 1681;
        end
        42: begin
          val <= 1764;
        end
        43: begin
          val <= 1849;
        end
        44: begin
          val <= 1936;
        end
        45: begin
          val <= 2025;
        end
        46: begin
          val <= 2116;
        end
        47: begin
          val <= 2209;
        end
        48: begin
          val <= 2304;
        end
        49: begin
          val <= 2401;
        end
        50: begin
          val <= 2500;
        end
        51: begin
          val <= 2601;
        end
        52: begin
          val <= 2704;
        end
        53: begin
          val <= 2809;
        end
        54: begin
          val <= 2916;
        end
        55: begin
          val <= 3025;
        end
        56: begin
          val <= 3136;
        end
        57: begin
          val <= 3249;
        end
        58: begin
          val <= 3364;
        end
        59: begin
          val <= 3481;
        end
        60: begin
          val <= 3600;
        end
        61: begin
          val <= 3721;
        end
        62: begin
          val <= 3844;
        end
        63: begin
          val <= 3969;
        end
        64: begin
          val <= 4096;
        end
        65: begin
          val <= 4225;
        end
        66: begin
          val <= 4356;
        end
        67: begin
          val <= 4489;
        end
        68: begin
          val <= 4624;
        end
        69: begin
          val <= 4761;
        end
        70: begin
          val <= 4900;
        end
        71: begin
          val <= 5041;
        end
        72: begin
          val <= 5184;
        end
        73: begin
          val <= 5329;
        end
        74: begin
          val <= 5476;
        end
        75: begin
          val <= 5625;
        end
        76: begin
          val <= 5776;
        end
        77: begin
          val <= 5929;
        end
        78: begin
          val <= 6084;
        end
        79: begin
          val <= 6241;
        end
        80: begin
          val <= 6400;
        end
        81: begin
          val <= 6561;
        end
        82: begin
          val <= 6724;
        end
        83: begin
          val <= 6889;
        end
        84: begin
          val <= 7056;
        end
        85: begin
          val <= 7225;
        end
        86: begin
          val <= 7396;
        end
        87: begin
          val <= 7569;
        end
        88: begin
          val <= 7744;
        end
        89: begin
          val <= 7921;
        end
        90: begin
          val <= 8100;
        end
        91: begin
          val <= 8281;
        end
        92: begin
          val <= 8464;
        end
        93: begin
          val <= 8649;
        end
        94: begin
          val <= 8836;
        end
        95: begin
          val <= 9025;
        end
        96: begin
          val <= 9216;
        end
        97: begin
          val <= 9409;
        end
        98: begin
          val <= 9604;
        end
        99: begin
          val <= 9801;
        end
        100: begin
          val <= 10000;
        end
        101: begin
          val <= 10201;
        end
        102: begin
          val <= 10404;
        end
        103: begin
          val <= 10609;
        end
        104: begin
          val <= 10816;
        end
        105: begin
          val <= 11025;
        end
        106: begin
          val <= 11236;
        end
        107: begin
          val <= 11449;
        end
        108: begin
          val <= 11664;
        end
        109: begin
          val <= 11881;
        end
        110: begin
          val <= 12100;
        end
        111: begin
          val <= 12321;
        end
        112: begin
          val <= 12544;
        end
        113: begin
          val <= 12769;
        end
        114: begin
          val <= 12996;
        end
        115: begin
          val <= 13225;
        end
        116: begin
          val <= 13456;
        end
        117: begin
          val <= 13689;
        end
        118: begin
          val <= 13924;
        end
        119: begin
          val <= 14161;
        end
        120: begin
          val <= 14400;
        end
        121: begin
          val <= 14641;
        end
        122: begin
          val <= 14884;
        end
        123: begin
          val <= 15129;
        end
        124: begin
          val <= 15376;
        end
        125: begin
          val <= 15625;
        end
        126: begin
          val <= 15876;
        end
        127: begin
          val <= 16129;
        end
        128: begin
          val <= 16384;
        end
        129: begin
          val <= 16641;
        end
        130: begin
          val <= 16900;
        end
        131: begin
          val <= 17161;
        end
        132: begin
          val <= 17424;
        end
        133: begin
          val <= 17689;
        end
        134: begin
          val <= 17956;
        end
        135: begin
          val <= 18225;
        end
        136: begin
          val <= 18496;
        end
        137: begin
          val <= 18769;
        end
        138: begin
          val <= 19044;
        end
        139: begin
          val <= 19321;
        end
        140: begin
          val <= 19600;
        end
        141: begin
          val <= 19881;
        end
        142: begin
          val <= 20164;
        end
        143: begin
          val <= 20449;
        end
        144: begin
          val <= 20736;
        end
        145: begin
          val <= 21025;
        end
        146: begin
          val <= 21316;
        end
        147: begin
          val <= 21609;
        end
        148: begin
          val <= 21904;
        end
        149: begin
          val <= 22201;
        end
        150: begin
          val <= 22500;
        end
        151: begin
          val <= 22801;
        end
        152: begin
          val <= 23104;
        end
        153: begin
          val <= 23409;
        end
        154: begin
          val <= 23716;
        end
        155: begin
          val <= 24025;
        end
        156: begin
          val <= 24336;
        end
        157: begin
          val <= 24649;
        end
        158: begin
          val <= 24964;
        end
        159: begin
          val <= 25281;
        end
        160: begin
          val <= 25600;
        end
        161: begin
          val <= 25921;
        end
        162: begin
          val <= 26244;
        end
        163: begin
          val <= 26569;
        end
        164: begin
          val <= 26896;
        end
        165: begin
          val <= 27225;
        end
        166: begin
          val <= 27556;
        end
        167: begin
          val <= 27889;
        end
        168: begin
          val <= 28224;
        end
        169: begin
          val <= 28561;
        end
        170: begin
          val <= 28900;
        end
        171: begin
          val <= 29241;
        end
        172: begin
          val <= 29584;
        end
        173: begin
          val <= 29929;
        end
        174: begin
          val <= 30276;
        end
        175: begin
          val <= 30625;
        end
        176: begin
          val <= 30976;
        end
        177: begin
          val <= 31329;
        end
        178: begin
          val <= 31684;
        end
        179: begin
          val <= 32041;
        end
        180: begin
          val <= 32400;
        end
        181: begin
          val <= 32761;
        end
        182: begin
          val <= 33124;
        end
        183: begin
          val <= 33489;
        end
        184: begin
          val <= 33856;
        end
        185: begin
          val <= 34225;
        end
        186: begin
          val <= 34596;
        end
        187: begin
          val <= 34969;
        end
        188: begin
          val <= 35344;
        end
        189: begin
          val <= 35721;
        end
        190: begin
          val <= 36100;
        end
        191: begin
          val <= 36481;
        end
        192: begin
          val <= 36864;
        end
        193: begin
          val <= 37249;
        end
        194: begin
          val <= 37636;
        end
        195: begin
          val <= 38025;
        end
        196: begin
          val <= 38416;
        end
        197: begin
          val <= 38809;
        end
        198: begin
          val <= 39204;
        end
        199: begin
          val <= 39601;
        end
        200: begin
          val <= 40000;
        end
        201: begin
          val <= 40401;
        end
        202: begin
          val <= 40804;
        end
        203: begin
          val <= 41209;
        end
        204: begin
          val <= 41616;
        end
        205: begin
          val <= 42025;
        end
        206: begin
          val <= 42436;
        end
        207: begin
          val <= 42849;
        end
        208: begin
          val <= 43264;
        end
        209: begin
          val <= 43681;
        end
        210: begin
          val <= 44100;
        end
        211: begin
          val <= 44521;
        end
        212: begin
          val <= 44944;
        end
        213: begin
          val <= 45369;
        end
        214: begin
          val <= 45796;
        end
        215: begin
          val <= 46225;
        end
        216: begin
          val <= 46656;
        end
        217: begin
          val <= 47089;
        end
        218: begin
          val <= 47524;
        end
        219: begin
          val <= 47961;
        end
        220: begin
          val <= 48400;
        end
        221: begin
          val <= 48841;
        end
        222: begin
          val <= 49284;
        end
        223: begin
          val <= 49729;
        end
        224: begin
          val <= 50176;
        end
        225: begin
          val <= 50625;
        end
        226: begin
          val <= 51076;
        end
        227: begin
          val <= 51529;
        end
        228: begin
          val <= 51984;
        end
        229: begin
          val <= 52441;
        end
        230: begin
          val <= 52900;
        end
        231: begin
          val <= 53361;
        end
        232: begin
          val <= 53824;
        end
        233: begin
          val <= 54289;
        end
        234: begin
          val <= 54756;
        end
        235: begin
          val <= 55225;
        end
        236: begin
          val <= 55696;
        end
        237: begin
          val <= 56169;
        end
        238: begin
          val <= 56644;
        end
        239: begin
          val <= 57121;
        end
        240: begin
          val <= 57600;
        end
        241: begin
          val <= 58081;
        end
        242: begin
          val <= 58564;
        end
        243: begin
          val <= 59049;
        end
        244: begin
          val <= 59536;
        end
        245: begin
          val <= 60025;
        end
        246: begin
          val <= 60516;
        end
        247: begin
          val <= 61009;
        end
        248: begin
          val <= 61504;
        end
        249: begin
          val <= 62001;
        end
        250: begin
          val <= 62500;
        end
        251: begin
          val <= 63001;
        end
        252: begin
          val <= 63504;
        end
        253: begin
          val <= 64009;
        end
        254: begin
          val <= 64516;
        end
        255: begin
          val <= 65025;
        end
      endcase
    end 
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = dataflow_lut.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from veriloggen import *
import veriloggen.dataflow as dataflow

def complex_add(x, y):
    a = x[0]
    b = x[1]
    c = y[0]
    d = y[1]
    return (a + c), (b + d)

def complex_sub(x, y):
    a = x[0]
    b = x[1]
    c = y[0]
    d = y[1]
    return (a - c), (b - d)

def complex_mult(x, y):
    a = x[0]
    b = x[1]
    c = y[0]
    d = y[1]
    # (a + jb) * (c + jd) = ac - bd + j(ad + bc)
    ac = a * c
    bd = b * d
    ad = a * d
    bc = b * c
    re = ac - bd
    im = ad + bc
    return re, im

def radix2(x, y, c):
    d0 = complex_add(x, y)
    d1 = complex_sub(x, y)
    r0 = d0 # as-is
    r1 = complex_mult(d1, c)
    return r0, r1

def mkRadix2(datawidth=32):
    din0 = (dataflow.Variable('din0re', width=datawidth),
            dataflow.Variable('din0im', width=datawidth))
    din1 = (dataflow.Variable('din1re', width=datawidth),
            dataflow.Variable('din1im', width=datawidth))
    cnst = (dataflow.Variable('cnstre', width=datawidth),
            dataflow.Variable('cnstim', width=datawidth))
    
    r0, r1 = radix2(din0, din1, cnst)
    
    r0[0].output('dout0re')
    r0[1].output('dout0im')
    r1[0].output('dout1re')
    r1[1].output('dout1im')
    
    rslt = list(r0) + list(r1)
    df = dataflow.Dataflow(*rslt)
    
    m = df.to_module('radix2')

    try:
        df.draw_graph()
    except:
        print('Dataflow graph could not be generated.', file=sys.stderr)

    return m

def mkTest(datawidth=32):
    m = Module('test')

    main = mkRadix2(datawidth)
    
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)

    clk = ports['CLK']
    rst = ports['RST']

    din0re = ports['din0re']
    din0im = ports['din0im']
    din1re = ports['din1re']
    din1im = ports['din1im']
    cnstre = ports['cnstre']
    cnstim = ports['cnstim']
    dout0re = ports['dout0re']
    dout0im = ports['dout0im']
    dout1re = ports['dout1re']
    dout1im = ports['dout1im']
    
    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    reset_done = m.Reg('reset_done', initval=0)
    reset_stmt = []
    reset_stmt.append( reset_done(0) )
    reset_stmt.append( din0re(2) )
    reset_stmt.append( din0im(0) )
    reset_stmt.append( din1re(1) )
    reset_stmt.append( din1im(0) )
    reset_stmt.append( cnstre(0) )
    reset_stmt.append( cnstim(1) )

    simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, reset_stmt, period=100)

    nclk = simulation.next_clock
    
    init.add(
        Delay(1000),
        reset_done(1),
        nclk(clk),
        Delay(10000),
        Systask('finish'),
    )

    send_fsm = FSM(m, 'send_fsm', clk, rst)
    send_fsm.goto_next(cond=reset_done)

    for i in range(10):
        send_fsm.goto_next()

    send_fsm.add( Systask('finish') )

    send_fsm.make_always()

    return m

if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    # run simulator (Icarus Verilog)
    sim = simulation.Simulator(test)
    rslt = sim.run() # display=False
    #rslt = sim.run(display=True)
    print(rslt)
